/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Config.h"
#include "debug.h"
#include "Error.h"
#include <errno.h>
#include "ConfigItem.h"

ConfigNodeList Config::nodes;

ConfigNodeList Config;
list<Configurable*> Config::Classes;
VisualInterface *Config::visual = 0;
App *Config::app = 0;
dstring Config::configfile;

ConfigProperty::ConfigProperty(const dstring&n, const dstring& v= dstring()):
    pair<const dstring,dstring>(n,v)
{
}
dstring &ConfigProperty::operator=(const dstring &a)
{
    second = a;
    return second;
}
dstring &ConfigProperty::operator=( int a)
{
    second = a;
    return second;
}

bool ConfigProperty::operator==(const dstring &a)
{
    return (a == second());
}

ConfigNode::ConfigNode(const dstring &n) :
_name(n), unique(0)
{
}
ConfigNode::ConfigNode(const dstring &n, int u) :
_name(n), unique(u)
{
}

void ConfigNode::NewPROPERTY(const ConfigProperty &c)
{
 //   ConfigPropertyList::const_iterator it;
 //   ConfigPropertyList::iterator f;
    ConfigPropertyList::iterator i;
    for (i = properties.begin(); i!=properties.end(); i++) {
	if ((*i).name() == c.name()) {
	    //            DEBUG(1,"Property found - not adding\n");
	    properties.erase( i );
            break;
	}
    }
    properties.add(c);
}
void ConfigNode::NewPROPERTY_IfNotFound(const ConfigProperty &c)
{
    ConfigPropertyList::iterator i;
    for (i = properties.begin(); i!=properties.end(); i++) {
	if ((*i).name() == c.name())
//            DEBUG(1,"Property found - not adding\n");
            return;
    }

    //   ConfigPropertyList::const_iterator it;
    //   ConfigPropertyList::iterator f;
  //  DEBUG(1,"Property not found - adding\n");
    properties.add(c);
}

ConfigProperty*ConfigNode::NewPROPERTY(const dstring &c)
{
    return (ConfigProperty*)&(*properties.add(ConfigProperty(c)));
}

void ConfigNode::DeletePROPERTY(const dstring&cname)
{
    ConfigPropertyList::iterator i;
    for (i = properties.begin(); i!=properties.end(); i++) {
	if ((*i).name() == cname) {
	    properties.erase(i);
            break;
	}
    }
}

ConfigNode *ConfigNode::NODE(const dstring&n)
{
    ConfigNodeList::iterator it;
    for (it=children.begin(); it!=children.end(); it++) {
	if ((*it)->name() == n)
            return (*it);
    }
    throw Error("No such node %s",n.Char());
}

ConfigProperty *ConfigNode::PROPERTY(const dstring &n)
{
    ConfigPropertyList::iterator i;
    for (i = properties.begin(); i!=properties.end(); i++) {
	if ((*i).name() == n)
            return (ConfigProperty*)&(*i);
    }
    throw PropertyNotFound();
}

ConfigNode *Config::NODE(const dstring &n)
{
    return nodes.NODE(n);
}

ConfigNode *ConfigNode::NewUniqueNODE(const dstring &n)
{
    ConfigNode *r;
    try {
	r = NODE(n);
    } catch (...) {
	r= new ConfigNode(n, true);
        r->setParent(this);
        children.insert(children.end(), r);
    }
    return r;
}

void ConfigNode::DeleteNODE_IfEmpty(const dstring &n)
{
    ConfigNode *r;
    try {
	r=NODE(n);
	if (r->properties.empty()) {
	    children.remove(r);
            delete(r);
	}
    } catch(...) {}
}
ConfigNode *ConfigNode::NewNODE_IfNotFound(const dstring&n)
{
    ConfigNode *r;
    try {
	r=NODE(n);
    } catch(...) {
	r = new ConfigNode(n, false);
	r->setParent(this);
        children.insert(children.end(), r);
    }
    return r;
}

ConfigNode *ConfigNode::NewNODE(const dstring &n)
{
    ConfigNode *r;
     try {
	 r = NODE(n);
	 if (!r->isUnique())
             throw Error("not unique");
    } catch (...) {
	 r= new ConfigNode(n);
	 r->setParent(this);
	 children.insert(children.end(), r);

    }
     return r;
}

ConfigNode *Config::NewNODE(const dstring &n)
{
    ConfigNode *r;
    try {
	 r = NODE(n);
	 if (!r->isUnique())
             throw Error("not unique");
    } catch (...) {

	r= new ConfigNode(n);
	r->setParent(0);
	nodes.insert(nodes.end(), r);
    }
    return r;
}

ConfigNode *Config::NewUniqueNODE(const dstring &n)
{
    ConfigNode *r;
    try {
	r = NODE(n);
    } catch (...) {
	r= new ConfigNode(n, true);
        r->setParent(0);
        nodes.insert(nodes.end(), r);
    }
    return r;
}

void ConfigNode::NewPROPERTY(const ConfigPropertyList &l)
{
}

void Config::RegisterClass(Configurable *c) {
    Classes.insert(Classes.end(),c);
}

ConfigNode *ConfigNodeList::NODE(const dstring&n)
{
    iterator it;
    for (it=begin(); it!=end(); it++)
    {
	if ((*it)->name() == n)
	    return (*it);
    }
    throw Error("Node %s not found", n.Char());
}
/*
ClassTypeHandler::Handler Config::GetClassHandler(const dstring &name)
{
    list<Configurable *>::iterator it;
    for (it=Classes.begin(); it!=Classes.end(); it++)
    {
	if ((*it)->Name() == name )
	    return &(*it)->CreateEntity;
    }
    throw Error("No such class entity \"%s\"",name.Char());
}
*/
Entity *Config::CallClassHandler(const dstring &name,
				 ConfigNode *node,
				 VisualInterface &visual)
{
    list<Configurable *>::iterator it;
    for (it=Classes.begin(); it!=Classes.end(); it++)
    {
	if ((*it)->Name() == name )
	    return (*it)->CreateEntity(node, visual);
    }
    throw Error("No such class entity \"%s\"",name.Char());
}

void Config::NewConfig(const dstring &d)
{
    configfile = d;

}
void Config::LoadFrom(const dstring &d)
{
    // Old-style
    char buffer[16386];
    ConfigBuilder builder(this);;
    XMLParser parser(&builder);

    FILE *f = fopen(d.Char(),"r");
    if (!f)
	throw Error("Cannot open file: %s\n",strerror(errno));
    do {
	fgets(buffer, 16384, f);
	if (!feof(f)) {
            parser.ParseChunk(dstring(buffer));
	}
    } while (!feof(f));
    parser.ParseFinish();

    fclose(f);
    configfile = d;
}

void ConfigBuilder::StartDocument()
{
}
void ConfigBuilder::EndDocument()
{
}
void ConfigBuilder::StartElement(const dstring &name, const dstringhash &p)
{
    ConfigNode *n;
    dstringhash::const_iterator it;

    if (start) {
	if (name!="monitor")
	    throw ::Error("Invalid tag, expected \"monitor\", got %s\n",
			name.Char());
	start=0;
        return;
    }

    n = new ConfigNode(name);
    if (currentnode) {
        currentnode = currentnode->NewNODE(name);
    } else {
        currentnode = config->NewNODE(name);
    }

    for (it = p.begin(); it!=p.end(); it++)
    {
	currentnode->NewPROPERTY(ConfigProperty((*it).first(),(*it).second()));
    }
}

void ConfigBuilder::EndElement(const dstring &name)
{
    if (!currentnode) {
        start =1;
    } else
    currentnode=currentnode->getParent();
}

void ConfigBuilder::Warning(const dstring&)
{
}
void ConfigBuilder::Error(const dstring &)
{
}
void ConfigBuilder::FatalError(const dstring &s)
{
    throw ::Error(s.Char());
}

void Config::Save()
{
    FILE *f;
    f= fopen(configfile.Char(), "w");
    DumpTree(f);
    fclose(f);
}

void Config::DumpTree(FILE*f)
{
    ConfigNodeList::const_iterator it;
    fprintf(f,"<?xml version=\"1.0\"?>\n<monitor>\n");
    for (it = nodes.begin(); it!=nodes.end(); it++)
	DumpTree(f, (*it));
    fprintf(f,"</monitor>");
}

void Config::DumpTree(FILE*f, ConfigNode *n)
{
    static int treedepth = 1;
    ConfigNodeList::const_iterator it;
//    ConfigPropertyList::const_iterator prop_it;
    int i;

    for (i=0; i<treedepth; i++)
	fprintf(f,"  ");
    treedepth++;

    fprintf(f,"<%s", n->name().Char());
    fprintf(f,"%s",n->prop().Show().Char());

    fprintf(f,">\n");

    for (it=n->Childs().begin();
	 it!=n->Childs().end();
	 it++) {
	Config::DumpTree(f, *it);
    }
    treedepth--;
    for (i=0; i<treedepth; i++)
	fprintf(f,"  ");
    
    fprintf(f,"</%s>\n",n->name().Char());
}


#ifdef HAVE_GNOME_XML2

char XMLParser::errmsg[512];

XMLParser::XMLParser(ConfigBuilder *cfg)
{
    xmlSAXHandler sh = {
	0, 0, 0, 0, 0,
	xmlGetEntity,
	0, 0, 0, 0, 0, 0,
	xmlStartDocument,
	xmlEndDocument,
	xmlStartElement,
	xmlEndElement,
	0, 0, 0, 0, 0,
	xmlWarning,
	xmlError,
	xmlFatalError,
	0, 0, 0 };
    context = xmlCreatePushParserCtxt(&sh, cfg, 0, 0, 0);
}

XMLParser::~XMLParser()
{
    if (context)
        xmlFreeParserCtxt(context);
}

void XMLParser::ParseChunk(const dstring &d)
{
    xmlParseChunk(context, d.Char(), d.length(), 0);
}

void XMLParser::ParseFinish()
{
    xmlParseChunk(context, NULL, 0, 0);
}

xmlEntity *XMLParser::xmlGetEntity(void *p, const xmlChar*c)
{
    return xmlGetPredefinedEntity(c);
}
void XMLParser::xmlStartDocument(void *p)
{
    ((ConfigBuilder*)p)->StartDocument();
}
void XMLParser::xmlEndDocument(void *p)
{
    ((ConfigBuilder*)p)->EndDocument();
}
void XMLParser::xmlStartElement(void *p, const xmlChar*c,
		     const xmlChar **cc)
{
    int i=0;
    dstringhash l;
    while (cc && cc[i]) {
	l[dstring((char*)cc[i])] = dstring((char*)cc[i+1]);
        i+=2;
    }
    ((ConfigBuilder*)p)->StartElement(dstring((char*)c), l);
}

void XMLParser::xmlEndElement(void *p, const xmlChar*c)
{
    ((ConfigBuilder*)p)->EndElement(dstring((char*)c));
}
void XMLParser::xmlWarning(void *p, const char *format, ...)
{
    va_list ap;
    va_start(ap, format);
    vsnprintf(errmsg, 512, format, ap);
    va_end(ap);
    ((ConfigBuilder*)p)->Warning(dstring(errmsg));
}
void XMLParser::xmlError(void *p, const char *format, ...)
{
    va_list ap;
    va_start(ap, format);
    vsnprintf(errmsg, 512, format, ap);
    va_end(ap);
    ((ConfigBuilder*)p)->Error(dstring(errmsg));
}

void XMLParser::xmlFatalError(void *p, const char *format, ...)
{
    va_list ap;
    va_start(ap, format);
    vsnprintf(errmsg, 512, format, ap);
    va_end(ap);
    ((ConfigBuilder*)p)->FatalError(dstring(errmsg));
}



#endif

#ifdef HAVE_LIBEXPAT

XMLParser::XMLParser(ConfigBuilder *cfg)
{
    xmlparser = (XML_Parser*)XML_ParserCreate(0); // Encoding here
    XML_SetElementHandler( xmlparser, xmlStartElement, xmlEndElement);
    XML_SetUserData(xmlparser, cfg);
}


void XMLParser::ParseChunk(const dstring &d)
{
    if (!XML_Parse(xmlparser, d.Char(), d.length(), 1)) {
        throw Error("Parse error in internal buffer, line %d: %s\n",
		XML_GetCurrentLineNumber(xmlparser),
		XML_ErrorString(XML_GetErrorCode(xmlparser)));
    }
}

void XMLParser::ParseFinish()
{
    XML_Parse(xmlparser, 0, 0, 0);
}

void XMLParser::xmlStartElement(void *p, const XML_Char*c,
		     const XML_Char **cc)
{
    int i=0;
    dstringhash l;
    while (cc && cc[i]) {
	l[dstring((char*)cc[i])] = dstring((char*)cc[i+1]);
        i+=2;
    }
    ((ConfigBuilder*)p)->StartElement(dstring((char*)c), l);
}

void XMLParser::xmlEndElement(void *p, const XML_Char*c)
{
    ((ConfigBuilder*)p)->EndElement(dstring((char*)c));
}

XMLParser::~XMLParser()
{
    if (xmlparser)
    XML_ParserFree(xmlparser);
}
#endif

