/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ConfigItem.h"
#include <stdio.h>


ConfigurableItemList::ConfigurableItemList()
{
}

ConfigurableItem::ConfigurableItem(const string &id, const string &name):
_id(id), _name(name)
{
}

CItemString::CItemString(const string &id, const string &name):
ConfigurableItem(id,name)
{
}

CItemBoolean::CItemBoolean(const string &id, const string &name):
ConfigurableItem(id,name)
{
}
CItemGroup::CItemGroup ( const string &name ):
ConfigurableItem("",name)
{
}

CItemGroup::~CItemGroup()
{
    iterator it;
    for (it=begin(); it!=end(); it++)
    {
	delete ( *it );
    }

}



CItemInteger::CItemInteger(const string &id, const string &name, int _min, int _max,
			  const string &un):
ConfigurableItem(id,name),  min(_min), max(_max), _units(un)
{

}


ConfigurableItemList &ConfigurableItemList::operator+(ConfigurableItemList *c)
{
 //   push_back(c);
    return *this;
}

ConfigurableItemList &ConfigurableItemList::operator+(ConfigurableItem *c)
{
    push_back(c);
    return *this;
}
ConfigurableItemList &ConfigurableItemList::operator+=(ConfigurableItem *c)
{
    push_back(c);
    return *this;
}

ConfigurableItemList::~ConfigurableItemList()
{
    iterator it;
    for (it=begin(); it!=end(); it++)
    {
	delete ( *it );
    }

}

ConfigurableItem *CItemGroup::findID(const string &id)
{
    iterator it;
    ConfigurableItem *item;
    for (it=begin(); it!=end(); it++)
    {
	item = (*it)->findID(id);
	if (item)
            return item;
    }
    return NULL;
}
ConfigurableItem *ConfigurableItemList::findID(const string &id)
{
    iterator it;
    ConfigurableItem *item;
    for (it=begin(); it!=end(); it++)
    {
	item = (*it)->findID(id);
	if (item)
            return item;
    }
    return NULL;
}


string &ConfigurableItemList::operator[](const string &id)
{
    ConfigurableItem *item = findID(id);
    if (item)
	return item->value();
    throw not_found();
}

int strint(const string &a)
{
    return atoi(a.c_str());
}
string intstr(int a)
{
    char tmp[128];
    snprintf(tmp,128, "%d",a);
    return string(tmp);
}

CItemGroup &CItemGroup::operator+=(ConfigurableItem *a)
{
    push_back(a);
    return *this;
}
