/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Entity.h"
#include "Led.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "Error.h"
#include "Visual.h"
#include "ConfigItem.h"
#include "Config.h"
class Led;

Entity::Entity(const dstring &h, const dstring& v, Led *w, Configurable *_c)
{
    widget = w;
    hostname = h;
    visualname = v;
    _class = _c;
    fail = 0;
}

void Entity::MoveTo(int xx,int yy)
{
    widget->MoveTo(xx,yy);
}


void Entity::IFail()
{
    if (!fail) widget->IFail(this);
    fail=1;
}

void Entity::IDoNotFail()
{
    if (fail) widget->IDoNotFail(this);
    fail =0;
}

void Entity::FindHost(const char *hostname, struct sockaddr_in &s)
{
    struct hostent *h=0;
    /* Addr is easier and faster, so... */
    memset(&s, 0, sizeof(struct sockaddr_in));
    s.sin_family = AF_INET;
        if (! inet_aton ( hostname,  &s.sin_addr) )
    {
	h = gethostbyname(hostname);
	if (!h)
	    throw Error("Host not Found");
	memcpy (&s.sin_addr.s_addr, &h->h_addr,
		h->h_length);
    }
}
Led &Entity::GetLed() { return *widget; }
void Entity::LeftClick() {};
void Entity::RightClick() {};
void Entity::MiddleClick() {};


ConfigurableItemList *Entity::Edit()
{
    return Class()->Edit(this);
}
