/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <unistd.h>
#include "App.h"
#include "Config.h"
#include "debug.h"
#include "classes/Classes.h"
#include <dlfcn.h>
#include <errno.h>
/*
 #include "visual/x11/X11App.h"
 #include "visual/gtk/GTKApp.h"
*/

#ifdef HAVE_GTK
#include "gtkconfig/GCMain.h"
#endif
#include <iostream>

#ifdef __linux__
#endif

int main(int argc, char **argv)
{
    Config c;
    DEBUG(5,"Starting\n");
    try {
#if 0 //def __linux__
	setresuid(getuid(),getuid(),getuid());
#else
	DEBUG(5,"Init classes\n");
	Classes::Init();
        DEBUG(5,"Drop privileges\n");
	setreuid(getuid(),getuid());
	setregid(getgid(),getgid());
#endif
	dstring filename;
	filename = getenv("HOME");
	filename += "/.monitoritrc";

	DEBUG(5,"Init app\n");
	App::Init();

	try {
	    try {
		c.LoadFrom(filename.Char());
	    } catch(...) {
		c.NewConfig(filename.Char());
	    }
	    //c.DumpTree(stderr);
	} catch(...) {
	    Error::Print();
	    throw ImmediateError("Cannot open configuration file %s\n"
				 "Please read \"example.config\"",filename.Char());
	}

	App *NetMon;

	DEBUG(5,"Initializing App\n");
	void *handler = dlopen(VISUAL_LIBDIR "/libmonitorit_visual_gtk.so",
			       RTLD_LAZY|RTLD_GLOBAL);
	if (handler==0) {
	    DEBUG(1, "Cannot open: %s\n", dlerror());
	    throw Error("Error");
	}

	void *fcn;

	fcn = dlsym(handler,"CreateVisual__FiPPc");
	if (fcn==0) {
	    DEBUG(1, "Cannot open: %s\n", dlerror());
	    throw Error("Error");
	}
	DEBUG(5,"Creating visual\n");
	NetMon = ((App*(*)(int,char**)) fcn)(argc, argv);
	DEBUG(5,"Running application\n");
	NetMon->Run();
	NetMon->Close();
        //delete (NetMon);
    } catch (...) {
	fprintf(stderr,"Upps: exception caught - ");
	Error::Print();
    }
}
