/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Pinger.h"
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/udp.h>
#include <arpa/inet.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include "Pinger.h"
#include "Config.h"
#include <string.h>
#include <errno.h>
#include "Led.h"
#include <sigc++/bind.h>
#include <sigc++/signal_system.h>
#include "App.h"

int PingerClass::fdsocket = 0;
int PingerClass::ident = 0;
int PingerClass::timeoutid = 0;
char PingerClass::outbuf[];
char PingerClass::inbuf[];
list<Pinger*> PingerClass::PingerItems = list<Pinger*>();

struct protoent *PingerClass::proto = 0;

//int PingerClass::cred, PingerClass::cyellow, PingerClass::ctime;

Configurable *createclass()
{
    return new PingerClass();
}


void PingerClass::SaveData()
{
    DEBUG(5,"Saving Pinger Data\n");
    Rehash();
}

PingerClass::PingerClass()
{
    DEBUG(5,"Creating class PINGER\n");
    ConfigNode *pinger = Config::NewUniqueNODE("pinger");
    pinger->NewUniqueNODE("timeout")->NewPROPERTY(ConfigProperty("yellow", "10"));
    pinger->NODE("timeout")->NewPROPERTY(ConfigProperty("red", "20"));
    pinger->NewUniqueNODE("time")->NewPROPERTY(ConfigProperty("ping", "10"));
    pinger->NewUniqueNODE("option")->NewPROPERTY(ConfigProperty("unreach", "0"));
    pinger->NODE("option")->NewPROPERTY(ConfigProperty("warn","true"));
    pinger->NODE("option")->NewPROPERTY(ConfigProperty("mute","false"));
    DEBUG(5,"Created sucessfully\n");
}

int PingerClass::rawsocket = PingerClass::AllocRawSocket();

int PingerClass::AllocRawSocket()
{
    int s;
    if (!(proto=getprotobyname("icmp"))) {
	throw ImmediateError("Unknown protocol icmp");
    }
    if ((s=socket(AF_INET,SOCK_RAW,proto->p_proto))<0) {
	if (errno==EPERM)
	    throw ImmediateError("This program must be run as root");
	else
	    throw ImmediateError("Cannot open socket:%s",strerror(errno));
    }
    ident =  getpid() & 0xFFFF;
    return s;
}



static int
    in_cksum(u_short *addr, int len)
{
    register int nleft = len;
    register u_short *w = addr;
    register int sum = 0;
    u_short answer = 0;

    while (nleft > 1)  {
	sum += *w++;
	nleft -= 2;
    }

    /* mop up an odd byte, if necessary */
    if (nleft == 1) {
	*(u_char *)(&answer) = *(u_char *)w ;
	sum += answer;
    }

    /* add back carry outs from top 16 bits to low 16 bits */
    sum = (sum >> 16) + (sum & 0xffff);     /* add hi 16 to low 16 */
    sum += (sum >> 16);                     /* add carry */
    answer = ~sum;                          /* truncate to 16 bits */
    return(answer);
}

void PingerClass::Init()
{
    int pingtime;

    if (fdsocket==0) {
        PingerClass::fdsocket = PingerClass::rawsocket;
	Config::GetVisual()->WatchFD(  PingerClass::fdsocket,
				       FDREAD,
				       SigC::slot( this, &PingerClass::FDHandler )
				    );
    }
    if (timeoutid == 0) {
	pingtime = Config::NODE("pinger")->NODE("time")->
	    PROPERTY("ping")->VALUE().Integer();
	timeoutid = Config::GetVisual()->RequestPeriodicTimeout(pingtime, 0, SigC::slot(this,&PingerClass::DoPoll));
    }
}

void PingerClass::Rehash()
{
    int pingtime;
    if (timeoutid != 0) {
	Config::GetVisual()->CancelTimeout(timeoutid);
	pingtime = Config::NODE("pinger")->NODE("time")->
	    PROPERTY("ping")->VALUE().Integer();
	timeoutid = Config::GetVisual()->RequestPeriodicTimeout(pingtime, 0, SigC::slot(this,&PingerClass::DoPoll));
    }
}



Pinger::Pinger(const char *hostname_, const char *visualname_, VisualInterface &v,
	       Led *l, PingerClass *p):
Entity( hostname_, visualname_, l, p)
{
    xmlnode = 0;
    p->Init();
    SetupHost();
    lastping = ClockTime();
    lastreply = ClockTime();
    SetState(PINGER_OFF);
}

void PingerClass::DoPoll()
{
    list<Pinger*>::iterator it;
    for (it=PingerClass::PingerItems.begin();
	 it!=PingerClass::PingerItems.end();
	 it++) {
	Pinger *p=(*it);
	TimeVal delay = p->Delay();

	if (delay > p->yellow) {
	    if (delay > p->red) {
		p->SetState(PINGER_FAIL);
	    }
	    else
		p->SetState(PINGER_WARN);
	}
	p->Poll();
    }
}
int Pinger::Poll()
{
   struct icmp  *icp;

   DEBUG(5,"Pinging %s...\n",
	 inet_ntoa(*(struct in_addr *)&sock.sin_addr.s_addr));

   icp = (struct icmp *)PingerClass::outbuf;
   
   lastping = ClockTime();

   icp->icmp_type = ICMP_ECHO;
   icp->icmp_code = 0;
   icp->icmp_cksum = 0;
   icp->icmp_seq = sequence++;
   icp->icmp_id = PingerClass::ident;                   /* ID */
   icp->icmp_cksum = in_cksum((u_short *)icp, DATALEN + 8);

   return sendto(PingerClass::fdsocket, (char *)PingerClass::outbuf, DATALEN + 8 , 0,
		 (struct sockaddr *)&sock,
		 sizeof(struct sockaddr));
}

void PingerClass::FDHandler(int fd)
{
    int count,hlen;
    struct sockaddr_in host;
    struct ip *iph;
    struct icmp *icp;
    size_t size=sizeof(struct sockaddr_in);

    count=recvfrom(PingerClass::fdsocket,(char*) PingerClass::inbuf, MAXPACKET, 0,
		   (struct sockaddr*)&host, &size);

    if ( count > 0) {
	iph=(struct ip*)PingerClass::inbuf;

	hlen = iph->ip_hl << 2;
	icp = (struct icmp *)(PingerClass::inbuf + hlen);
	switch (icp->icmp_type) {
	case ICMP_ECHOREPLY:
	    if (icp->icmp_id == PingerClass::ident) {

		list<Pinger*>::iterator it;

		for (it=PingerClass::PingerItems.begin();
		     it!=PingerClass::PingerItems.end(); it++) {

		    if ((*it)->Match (host))
			(*it)->GotReply(host);
		}
	    }
	    break;
	case ICMP_UNREACH:
	    if (icp->icmp_code == ICMP_UNREACH_HOST ||
		icp->icmp_code == ICMP_UNREACH_NET) {

		struct ip *ip2h = (struct ip*) (PingerClass::inbuf + hlen + 8);
		list<Pinger*>::iterator it;
		for (it=PingerClass::PingerItems.begin();
		     it!=PingerClass::PingerItems.end(); it++) {
		    if ((*it)->Match (ip2h->ip_dst))
			(*it)->GotUnreach(host);
		}
	    }
            break;
	}


    } else {
	DEBUG(2,"Received %d bytes from socket %d.. strange isn't it ?\n",
		count, PingerClass::fdsocket);
    }
}


void Pinger::SetState(PingerStateType state)
{
    currentstate = state;
    SetState();
}

void Pinger::SetState()
{
    switch (currentstate) {
    case PINGER_OK:
	if (muted) {
	    SetColor(LED_BLUE);
	    SetFlash(0);
	} else {
	    SetColor(LED_GREEN);
            SetFlash(0);
	}
	IDoNotFail();
        break;
    case PINGER_FAIL:
	if (muted) {
	    SetColor(LED_PURPLE);
	    SetFlash(0);
            IDoNotFail();
	} else {
	    if (warn) {
		SetColor(LED_RED);
		SetFlash(1);
		IFail();
	    } else {
		SetColor(LED_OFF);
		SetFlash(0);
                IDoNotFail();
	    }
	}
        break;
    case PINGER_OFF:
	SetColor(LED_OFF);
	SetFlash(0);
	IDoNotFail();
	break;
    case PINGER_WARN:
	if (muted)
	    SetColor(LED_BLUE);
	else SetColor(LED_YELLOW);
        IDoNotFail();
    }
}

void Pinger::GotReply(struct sockaddr_in &host)
{
    lastreply = ClockTime();

    DEBUG(5, "Got reply from %s\n",
	    inet_ntoa(*(struct in_addr *)&host.sin_addr.s_addr));

    SetState(PINGER_OK);
}

void Pinger::GotUnreach(struct sockaddr_in &host)
{
    if (unreach) {
	lastreply = TimeVal(0,0) ;
        SetState(PINGER_FAIL);
    }
}

int Pinger::SetupHost()
{
    struct hostent *h=0;
    /* Addr is easier and faster, so... */
    memset(&sock, 0, sizeof(struct sockaddr_in));

    sock.sin_family = AF_INET;
    if ( inet_aton(hostname.Char(),&sock.sin_addr) == 0)
    {
	h = gethostbyname(hostname.Char());
	if (!h)
	    throw Error("Invalid Host: %s\n",hostname.Char());
	bcopy(h->h_addr, &sock.sin_addr.s_addr,
		h->h_length);
    }

    list<Pinger*>::const_iterator it;
    for (it = PingerClass::PingerItems.begin();
	 it!= PingerClass::PingerItems.end();
	 it++) {
	if ((*it)==this)
	    break;
    }

    if (it== PingerClass::PingerItems.end()) {
	PingerClass::PingerItems.insert(PingerClass::PingerItems.end(), this);
    } else {
        DEBUG(5,"Altering pinger host\n");
    }
    lastreply = ClockTime();
    return 1;
}

void Pinger::Setup(ConfigNode *root)
{
    try {
	yellow = TimeVal(root->NODE("timeout")->PROPERTY("yellow")->
			    VALUE().Integer(), 0);
    } catch(...){
	yellow = TimeVal(Config::NODE("pinger")->NODE("timeout")->
			    PROPERTY("yellow")->VALUE().Integer(),0);
    }
    try {
	red = TimeVal(root->NODE("timeout")->PROPERTY("red")->
			 VALUE().Integer(), 0);
    } catch(...){
	red = TimeVal(Config::NODE("pinger")->NODE("timeout")->
			 PROPERTY("red")->VALUE().Integer(),0);
    }
    try {
	unreach = root->NODE("option")->PROPERTY("unreach")->
	    VALUE().Integer();
    } catch (...) {
	unreach = Config::NODE("pinger")->NODE("option")->
	    PROPERTY("unreach")->VALUE().Integer();
    }
    try {
	warn = (root->NODE("option")->PROPERTY("warn")->
	    VALUE() == "true");
    } catch (...) {
	warn = (Config::NODE("pinger")->NODE("option")->
		PROPERTY("warn")->VALUE() == "true");
    }
    try {
	muted = (root->NODE("option")->PROPERTY("mute")->
	    VALUE() == "true");
    } catch (...) {
	muted = (Config::NODE("pinger")->NODE("option")->
		PROPERTY("mute")->VALUE() == "true");
    }

    /* Reset defaults */
    SetState(PINGER_OFF);
}

Entity *PingerClass::CreateEntity(ConfigNode *root, VisualInterface &v)
{
    Pinger *p;
    try {
	p = new Pinger( root->NODE("host")->PROPERTY("address")->VALUE().Char(),
			root->PROPERTY("name")->VALUE().Char(), v, v.NewLed(), this );
        p->Setup(root);
	DEBUG(5,"Pinger %s warn=%d\n",p->visualname().Char(),p->warn);
	p->xmlnode = root;
        return p;
    } catch(...) {
	Error::Print();
        throw Error("Unable to create host, check your configuration");
    }
    
    return 0;
}

void Pinger::SetColor(LedColor c)
{
    dynamic_cast<Led*>(widget)->SetColor(c);
}
void Pinger::SetFlash(int c)
{
    dynamic_cast<Led*>(widget)->SetFlash(c);
}


Pinger::~Pinger()
{
    list<Pinger*>::iterator it;
    for (it=PingerClass::PingerItems.begin();
	 it!=PingerClass::PingerItems.end();
	 it++)
	if ((*it)==this)
	    PingerClass::PingerItems.erase(it);
}

ConfigurableItemList *PingerClass::Edit(Entity *e)
{
    ConfigurableItemList *list = Add();
    Pinger *p = static_cast<Pinger*> (e);

    (*list)["name"] = p->VisualName();
    (*list)["hostname"] = p->HostName();
    try {
	ConfigProperty *ovy = p->NODE()->NODE("timeout")->PROPERTY("yellow");
        (*list)["yflag"] = "true";
	(*list)["yvalue"] = ovy->VALUE();
    } catch(...) {
	(*list)["yflag"] = "false";
	(*list)["yvalue"] = Config::NODE("pinger")->NODE("timeout")->PROPERTY("yellow")->VALUE();
    }

    try {
	ConfigProperty *ovy = p->NODE()->NODE("timeout")->PROPERTY("red");
        (*list)["rflag"] = "true";
	(*list)["rvalue"] = ovy->VALUE();
    } catch(...) {
	(*list)["rflag"] = "false";
	(*list)["rvalue"] = Config::NODE("pinger")->NODE("timeout")->PROPERTY("red")->VALUE();
    }

    try {
	ConfigProperty *ovw = p->NODE()->NODE("option")->PROPERTY("warn");
	(*list)["warnflag"] = "true";
	(*list)["warn"] = ovw->VALUE();
    } catch(...) {
	(*list)["warnflag"] = "false";
    }

    try {
	ConfigProperty *ovw = p->NODE()->NODE("option")->PROPERTY("mute");
	(*list)["muteflag"] = "true";
	(*list)["mute"] = ovw->VALUE();
    } catch(...) {
	(*list)["muteflag"] = "false";
    }

    return list;
}
ConfigurableItemList *PingerClass::Add()
{
    
    ConfigurableItemList *list = new ConfigurableItemList();

    (*list)+= new CItemString("name", "Name");
    (*list)+= new CItemString("hostname", "Hostname");

    CItemGroup *group =  new CItemGroup("Override defaults");

    
    (*group)+= new CItemDependable<CItemInteger> (
						  new CItemInteger("yvalue", "", 0, 1000, "seconds"),
						  new CItemBoolean("yflag", "Yellow Timeout")
						 );
    (*group)+= new CItemDependable<CItemInteger> (
						  new CItemInteger("rvalue", "", 0, 1000, "seconds"),
						  new CItemBoolean("rflag", "Red Timeout")
						 );
    (*group)+= new CItemDependable<CItemBoolean> (
						  new CItemBoolean("mute", "Start Muted"),
						  new CItemBoolean("muteflag", "Mute Flag")
						 );
    (*group)+= new CItemDependable<CItemBoolean> (
						  new CItemBoolean("warn", "Don't warn"),
						  new CItemBoolean("warnflag", "Warn Flag")
						 );

    (*list)+=group;

    
    
    return list;
}

ConfigurableItemList *PingerClass::Config()
{
    
    ConfigurableItemList *list = new ConfigurableItemList();
    ConfigNode *pinger = Config::NODE("pinger");

    (*list)+= new CItemInteger("yvalue", "Yellow Timeout", 0, 1000, "seconds");
    (*list)+= new CItemInteger("rvalue", "Red Timeout", 0, 1000, "seconds");
    (*list)+= new CItemInteger("pvalue", "Ping Interval", 0, 1000, "seconds");

    // defaults

    (*list)["yvalue"] = pinger->NODE("timeout")->PROPERTY("yellow")->VALUE();
    (*list)["rvalue"] = pinger->NODE("timeout")->PROPERTY("red")->VALUE();
    (*list)["pvalue"] = pinger->NODE("time")->PROPERTY("ping")->VALUE();

    return list;
}


int PingerClass::ApplyConfig( ConfigurableItemList *list )
{
    ConfigNode *pinger = Config::NODE("pinger");
    *(pinger->NODE("timeout")->PROPERTY("yellow")) = (*list)["yvalue"];
    *(pinger->NODE("timeout")->PROPERTY("red")) = (*list)["rvalue"];
    *(pinger->NODE("time")->PROPERTY("ping")) = (*list)["pvalue"];
    Rehash();
}

void Pinger::LeftClick()
{
    DEBUG(5,"Left clicked on a pinger host\n");
    if (muted) muted=0;
    else muted=1;
    SetState();
}

int PingerClass::Save(ConfigurableItemList &list)
{

    DEBUG(5,"Saving data\n");

    ConfigNode *ent = Config::NewNODE("entity");

    *ent->NewPROPERTY("type")=PingerName;
    *ent->NewPROPERTY("name")= list["name"];

    *ent->NewNODE("host")->NewPROPERTY("address")=
	list["hostname"];

    if (list["yflag"] == "true") {
	*ent->NewNODE_IfNotFound("timeout")->NewPROPERTY("yellow") =
	    strint(list["yvalue"]);
    }
    if (list["rflag"] == "true") {
	*ent->NewNODE_IfNotFound("timeout")->NewPROPERTY("red") =
	    strint(list["rvalue"]);
    }

    if (list["muteflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("mute")=
	    list["mute"];
    }

    if (list["warnflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("warn")=
	    list["warn"];
    }
    try {
	Config::GetApp()->Add( *Config::CallClassHandler( PingerName,
							  ent, *Config::GetVisual()));
    } catch (...) {

	delete(ent);
	return 0;
    }

    return 1;
}



int PingerClass::ApplyEdit( ConfigurableItemList *list, Entity *entity)
{
    DEBUG(1,"Saving Data\n");

    DEBUG(5,"Setup Host\n");
    Pinger *cp =static_cast<Pinger*>(entity);

    ConfigNode *ent=static_cast<Pinger*>(entity)->NODE();

    
    cp->hostname = (*list)["hostname"];

    static_cast<Pinger*>(entity)->SetupHost();

    *ent->NODE("host")->PROPERTY("address")=
	(*list)["hostname"];

    if ((*list)["yflag"] == "true" ) {
	// Create if not found
	*ent->NewNODE_IfNotFound("timeout")->NewPROPERTY("yellow") =
	    strint((*list)["yvalue"]) ;
    } else {
	try {
	    ent->NODE("timeout")->DeletePROPERTY("yellow");
	} catch(...) {}
    }

    if ((*list)["rflag"] == "true" ) {
	    // Create if not found
	*ent->NewNODE_IfNotFound("timeout")->NewPROPERTY("red") =
	    strint((*list)["rvalue"]) ;
    } else {
	try {
	    ent->NODE("timeout")->DeletePROPERTY("red");
	} catch(...) {}
    }

    if ((*list)["muteflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("mute")=
	    (*list)["mute"];
    } else {
	try {
	    ent->NODE("option")->DeletePROPERTY("mute");
	} catch(...) {}
    }

    if ((*list)["warnflag"] == "true" ) {
	    *ent->NewNODE_IfNotFound("option")->NewPROPERTY("warn")=
		(*list)["warn"];
    } else {
	try {
	    ent->NODE("option")->DeletePROPERTY("warn");
	} catch(...) {}
    }


    ent->DeleteNODE_IfEmpty("timeout");
    ent->DeleteNODE_IfEmpty("option");
    DEBUG(5,"This entity global setup\n");
    static_cast<Pinger*>(entity)->Setup(ent);

    return 1;

}

int PingerClass::ApplyAdd( ConfigurableItemList *list)
{
    DEBUG(5,"Saving data\n");

    ConfigNode *ent = Config::NewNODE("entity");

    *ent->NewPROPERTY("type")=PingerName;
    *ent->NewPROPERTY("name")= (*list)["name"];

    *ent->NewNODE("host")->NewPROPERTY("address")=
	(*list)["hostname"];

    if ((*list)["yflag"] == "true") {
	*ent->NewNODE_IfNotFound("timeout")->NewPROPERTY("yellow") =
	    strint((*list)["yvalue"]);
    }
    if ((*list)["rflag"] == "true") {
	*ent->NewNODE_IfNotFound("timeout")->NewPROPERTY("red") =
	    strint((*list)["rvalue"]);
    }

    if ((*list)["muteflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("mute")=
	    (*list)["mute"];
    }

    if ((*list)["warnflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("warn")=
	    (*list)["warn"];
    }
    try {
	Config::GetApp()->Add( *Config::CallClassHandler( PingerName,
							  ent, *Config::GetVisual()));
    } catch (...) {
	delete(ent);
        throw;
    }
    return 0;
}

