/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __PINGER_H__
#define __PINGER_H__

#include "config.h"
#include "Entity.h"
#include "TimeVal.h"
#include "Config.h"
#include "Visual.h"
#include "Led.h"
#include <sys/types.h>
#include <netinet/in.h>
#include "ConfigItem.h"

typedef enum {
	PINGER_OK,
        PINGER_WARN,
	PINGER_FAIL,
        PINGER_OFF
} PingerStateType;

const int DATALEN = 8;
const int MAXPACKET = 64;

const dstring PingerName("ping");
const dstring PingerVisualName("Pinger");

class PingerClass: public Configurable
{
public:
    friend class Pinger;
    PingerClass();
    virtual Entity *CreateEntity(ConfigNode*, VisualInterface&);
    virtual const dstring &Name() const { return PingerName; }
    virtual const dstring &VisualName() const { return PingerVisualName; }
    virtual void Rehash();
    void Init();
    void FDHandler(int fd);
    void DoPoll(void);
    void SaveData();
    int Save(ConfigurableItemList &list);
    virtual ConfigurableItemList *Edit(Entity *);
    virtual ConfigurableItemList *Config();
    virtual ConfigurableItemList *Add();

    virtual int ApplyEdit( ConfigurableItemList *, Entity *);
    virtual int ApplyConfig( ConfigurableItemList *);
    virtual int ApplyAdd( ConfigurableItemList *);

    static int AllocRawSocket();
protected:
    static int fdsocket;
    static int rawsocket;
    static int timeoutid;
    static list<Pinger*> PingerItems;
    static struct protoent *proto;
public:
    static int ident;
    static char outbuf[8+DATALEN];
    static char inbuf[MAXPACKET];
};

class Pinger: public Entity
{
public:

    Pinger(const char *hostname, const char *visualname,VisualInterface&,
	   Led *l, PingerClass *p);
    ~Pinger();
    
    int FDChanged(int fd,  int type);
    
    virtual const dstring ClassName() { return dstring("ping"); }
    int Poll();
    int Match(struct sockaddr_in &s) {
        return s.sin_addr.s_addr == sock.sin_addr.s_addr;
    }
    int Match(struct in_addr addr) {
        return addr.s_addr == sock.sin_addr.s_addr;
    }

    void GotReply(struct sockaddr_in &host);
    void GotUnreach(struct sockaddr_in &host);
    TimeVal Delay() { return lastping - lastreply; }
    void Setup(ConfigNode *);
    Entity* CreateEntity(ConfigNode *,VisualInterface&);
    void SetColor(LedColor c);
    void SetFlash(int c);


    friend class PingerClass;
    friend class PingerEntityWindow;
    ConfigNode *NODE() { return xmlnode; }
    virtual void LeftClick();
    /*
    virtual void RightClick();
    virtual void MiddleClick();
    */
    void SetState(PingerStateType state);
    void SetState();
protected:
    int SetupHost();
    struct sockaddr_in sock;
    int unreach;
    int sequence;
    TimeVal lastping;
    TimeVal lastreply;
    //static TimeVal def_yellow;
    //static TimeVal def_red;
    TimeVal yellow,red;
    int warn, muted;
    PingerStateType currentstate;
    ConfigNode *xmlnode;
    int fail;
};



#endif
