/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Tcp.h"
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/udp.h>
#include <arpa/inet.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include "Config.h"
#include <string.h>
#include <errno.h>
#include "Led.h"
#include "App.h"

list<Tcp*> TcpClass::TcpItems = list<Tcp*>();

struct protoent *TcpClass::proto = 0;

Configurable *createclass()
{
    return new TcpClass();
}


TcpClass::TcpClass()
{
    DEBUG(5,"Creating class TCP\n");
    ConfigNode *tcpnode = Config::NewUniqueNODE("tcp");
    tcpnode->NewUniqueNODE("time")->NewPROPERTY_IfNotFound(ConfigProperty("poll", "60"));
    tcpnode->NewUniqueNODE("option")->NewPROPERTY_IfNotFound(ConfigProperty("warn","true"));
    tcpnode->NODE("option")->NewPROPERTY_IfNotFound(ConfigProperty("mute","false"));
}

void TcpClass::Init()
{
    int polltime;
}

void TcpClass::Rehash()
{
}

Tcp::Tcp(const char *hostname_, int rport, const char *visualname_, VisualInterface &v,
	       Led *l, TcpClass *p):
Entity( hostname_, visualname_, l,p)
{
    xmlnode = 0;
    fdsocket = 0;
    port = rport;
    warn = 1;
    muted = 0;

    TcpClass::Init();
    DEBUG(1,"Create TCP\n");
    SetupHost();

    SetState(TCP_OFF);
}

void TcpClass::DoPoll(void)
{
    list<Tcp*>::iterator it;
    for (it=TcpClass::TcpItems.begin();
	 it!=TcpClass::TcpItems.end();
	 it++) {
	Tcp *p=(*it);
	p->Poll();
    }
}
void Tcp::Poll()
{
    int reuse=1;
    if (fdsocket) {
        return ;
    }

    fdsocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

    if (fdsocket<=0) {
	DEBUG(1,"Cannot allocate socket\n");
        return ;
    }

    /* Setup non-blocking IO */

    if (fcntl (fdsocket, F_SETFL , fcntl(fdsocket, F_GETFL,0) | O_NONBLOCK )<0) {
	DEBUG(1,"fcntl setting nonblocking for tcp socket");
        shutdown(fdsocket, SHUT_RDWR);
	return ;
    }

    /* Set flags */
    if (setsockopt(fdsocket, SOL_SOCKET,
		   SO_REUSEADDR, (void*)&reuse,
		   sizeof(int))<0) {
	DEBUG(5,"Cannot set Reuseaddr: %s\n", strerror(errno));
    }

    DEBUG(5,"Connecting to %s, port %d\n",
	  inet_ntoa(*(struct in_addr *)&sock.sin_addr.s_addr), port);

    if (connect(fdsocket, (struct sockaddr*)&sock, sizeof(struct sockaddr_in))<0) {
	switch(errno) {
	case EISCONN:
	case EALREADY:
	    DEBUG(6,"Already connected, shutting down..\n");
	    shutdown(fdsocket, 3);
	    break;
	case EINPROGRESS:
            DEBUG(6,"In progress, waiting..\n");
	    watchfdid = Config::GetVisual()->WatchFD(fdsocket,
						     FDWRITE ,
						     SigC::slot ( this, &Tcp::FDHandler)
						     );
	    break;
	default:
	    DEBUG(6,"Error, setting failure\n");
            perror("connect");
            SetState(TCP_FAIL);
	}
    } else {
	DEBUG(5,"Connect OK\n");
	GotConnect();
    }
}

int TcpClass::StaticFDHandler(int fd,FDTYPE type, void *data)
{
    Tcp *p = (Tcp*)data;

    p->GotConnect();

}

void Tcp::FDHandler(int fd)
{
    GotConnect();
}


void Tcp::SetState(TcpStateType state)
{
    currentstate = state;
    SetState();
}

void Tcp::SetState()
{
    switch (currentstate) {
    case TCP_OK:
	if (muted) {
	    SetColor(LED_BLUE);
	    SetFlash(0);
	} else {
	    SetColor(LED_GREEN);
            SetFlash(0);
	}
	IDoNotFail();
        break;
    case TCP_FAIL:
	DEBUG(2,"warn %d muted %d\n",
              warn,muted);
	if (muted) {
	    SetColor(LED_PURPLE);
	    SetFlash(0);
            IDoNotFail();
	} else {
	    if (warn) {
		SetColor(LED_RED);
		SetFlash(1);
		IFail();
	    } else {
		SetColor(LED_OFF);
		SetFlash(0);
                IDoNotFail();
	    }
	}
        break;
    case TCP_OFF:
	SetColor(LED_OFF);
	SetFlash(0);
	IDoNotFail();
	break;
    case TCP_WARN:
	if (muted)
	    SetColor(LED_BLUE);
	else SetColor(LED_YELLOW);
        IDoNotFail();
    }
}

void Tcp::GotConnect()
{
    int failflag= 0;
    socklen_t failflagsize=sizeof(int);

    if (getsockopt(fdsocket,
		   SOL_SOCKET, SO_ERROR, &failflag, &failflagsize)<0) {
        perror("getsockopt");
    }
    if (failflag) {
	DEBUG(5, "Cannot TCP connect %s, port %d\n",
	      inet_ntoa(*(struct in_addr *)&sock.sin_addr.s_addr), port);
        perror("connect");
	SetState(TCP_FAIL);
    } else {
	DEBUG(5, "Got TCP connect for %s, port %d\n",
	      inet_ntoa(*(struct in_addr *)&sock.sin_addr.s_addr), port);

	SetState(TCP_OK);
    }
    Config::GetVisual()->StopWatch( fdsocket );
    close( fdsocket );
    fdsocket = 0;
}


int Tcp::SetupHost()
{
    struct hostent *h=0;

    
    /* Addr is easier and faster, so... */
    memset(&sock, 0, sizeof(struct sockaddr_in));

    sock.sin_family = AF_INET;

    if ( inet_aton(hostname.Char(),&sock.sin_addr) == 0)
    {
	h = gethostbyname(hostname.Char());
	if (!h)
	    throw Error("Invalid Host: %s\n",hostname.Char());
	bcopy(h->h_addr, &sock.sin_addr.s_addr,
	      h->h_length);
    }

    sock.sin_port = htons(Port());

    list<Tcp*>::const_iterator it;
    for (it = TcpClass::TcpItems.begin();
	 it!= TcpClass::TcpItems.end();
	 it++) {
	if ((*it)==this)
	    break;
    }

    if (it== TcpClass::TcpItems.end()) {
	TcpClass::TcpItems.insert(TcpClass::TcpItems.end(), this);
    } else {
        DEBUG(5,"Altering tcp host\n");
    }
    //    lastreply = ClockTime();

    return 1;
}

void TcpClass::Edit(const dstring &name)
{
    list<Tcp*>::iterator it;
    for (it=TcpItems.begin();
	 it!=TcpItems.end();
	 it++) {
	if ((*it)->VisualName() == name)
            (*it)->Edit();
    }
}

void TcpClass::AddEntity()
{
}

void Tcp::Setup(ConfigNode *root)
{
    try {
	warn = root->NODE("option")->PROPERTY("warn")->
	    VALUE() == "true" ;
    } catch (...) {
	warn = Config::NODE("tcp")->NODE("option")->
	    PROPERTY("warn")->VALUE() == "true";
    }
    try {
	muted = root->NODE("option")->PROPERTY("mute")->
	    VALUE() == "true";
    } catch (...) {
	muted = Config::NODE("tcp")->NODE("option")->
	    PROPERTY("mute")->VALUE() == "true";
    }
    int polltime;
    try {
	polltime = root->NODE("time")->
	    PROPERTY("poll")->VALUE().Integer();
    } catch (...) {
	polltime = Config::NODE("tcp")->NODE("time")->
	    PROPERTY("poll")->VALUE().Integer();
    }
    timeoutid = Config::GetVisual()->RequestPeriodicTimeout(polltime,
							    0,
							    SigC::slot(this,&Tcp::Poll)
							    );

    /* Reset defaults */
    SetState(TCP_OFF);
}

Entity *TcpClass::CreateEntity(ConfigNode *root, VisualInterface &v)
{
    Tcp *p;
    try {
	p = new Tcp( root->NODE("host")->PROPERTY("address")->VALUE().Char(),
                     root->NODE("host")->PROPERTY("port")->VALUE().Integer(),
			root->PROPERTY("name")->VALUE().Char(), v, v.NewLed(),this );
        p->Setup(root);
	DEBUG(5,"Tcp %s warn=%d\n",p->visualname().Char(),p->warn);
	p->xmlnode = root;
        return p;
    } catch(Error e) {
	//Error::Print();
	throw Error("Unable to create host: %s", e.error);
    } catch(...) {
	throw Error("Unable to create host");
    }
    
    return 0;
}

void Tcp::SetColor(LedColor c)
{
    dynamic_cast<Led*>(widget)->SetColor(c);
}
void Tcp::SetFlash(int c)
{
    dynamic_cast<Led*>(widget)->SetFlash(c);
}


Tcp::~Tcp()
{
    list<Tcp*>::iterator it;
    for (it=TcpClass::TcpItems.begin();
	 it!=TcpClass::TcpItems.end();
	 it++)
	if ((*it)==this)
	    TcpClass::TcpItems.erase(it);
}

void TcpClass::SaveData()
{
}

ConfigurableItemList *TcpClass::Config()
{
    
    ConfigurableItemList *list = new ConfigurableItemList();
    ConfigNode *tcp = Config::NODE("tcp");
    (*list)+= new CItemInteger("pvalue", "Poll Interval", 0, 1000, "seconds");
    (*list)["pvalue"] = tcp->NODE("time")->PROPERTY("poll")->VALUE();
    return list;
}

int TcpClass::ApplyConfig( ConfigurableItemList *list )
{
    ConfigNode *tcp = Config::NODE("tcp");
    *(tcp->NODE("time")->PROPERTY("poll")) = (*list)["pvalue"];
    Rehash();
}


ConfigurableItemList *TcpClass::Add()
{
    
    ConfigurableItemList *list = new ConfigurableItemList();

    (*list)+= new CItemString("name", "Name");
    (*list)+= new CItemString("hostname", "Hostname");
    (*list)+= new CItemInteger("port", "Port", 1, 65535);

    CItemGroup *group =  new CItemGroup("Override defaults");

    
    (*group)+= new CItemDependable<CItemInteger> (
						  new CItemInteger("pvalue", "", 0, 1000, "seconds"),
						  new CItemBoolean("pflag", "Poll Interval")
                                                  );
    (*group)+= new CItemDependable<CItemBoolean> (
						  new CItemBoolean("mute", "Start Muted"),
						  new CItemBoolean("muteflag", "Mute Flag")
						 );
    (*group)+= new CItemDependable<CItemBoolean> (
						  new CItemBoolean("warn", "Don't warn"),
						  new CItemBoolean("warnflag", "Warn Flag")
						 );

    (*list)+=group;
    return list;
}

int TcpClass::ApplyAdd( ConfigurableItemList *list)
{
    DEBUG(5,"Saving data -ADD\n");

    ConfigNode *ent = Config::NewNODE("entity");

    *ent->NewPROPERTY("type")=TcpName;
    *ent->NewPROPERTY("name")= (*list)["name"];

    *ent->NewNODE("host")->NewPROPERTY("address")=
	(*list)["hostname"];

    *ent->NODE("host")->NewPROPERTY("port")=
	(*list)["port"];

    if ((*list)["muteflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("mute")=
	    (*list)["mute"];
    }
    if ((*list)["pflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("time")->NewPROPERTY("poll")=
	    (*list)["pvalue"];
    }

    if ((*list)["warnflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("warn")=
	    (*list)["warn"];
    }
    try {
	Config::GetApp()->Add( *Config::CallClassHandler( TcpName,
							  ent, *Config::GetVisual()));
    } catch (...) {
	delete(ent);
        throw;
    }
    return 0;
}

ConfigurableItemList *TcpClass::Edit(Entity *e)
{
    ConfigurableItemList *list = Add();
    Tcp *p = static_cast<Tcp *> (e);

    (*list)["name"] = p->VisualName();
    (*list)["hostname"] = p->HostName();
    (*list)["port"] = intstr( p->Port() );
    try {
	ConfigProperty *ovw = p->NODE()->NODE("option")->PROPERTY("warn");
	(*list)["warnflag"] = "true";
	(*list)["warn"] = ovw->VALUE();
    } catch(...) {
	(*list)["warnflag"] = "false";
    }
    try {
	ConfigProperty *ovw = p->NODE()->NODE("time")->PROPERTY("poll");
	(*list)["pflag"] = "true";
	(*list)["pvalue"] = ovw->VALUE();
    } catch(...) {
	(*list)["pflag"] = "false";
    }


    try {
	ConfigProperty *ovw = p->NODE()->NODE("option")->PROPERTY("mute");
	(*list)["muteflag"] = "true";
	(*list)["mute"] = ovw->VALUE();
    } catch(...) {
	(*list)["muteflag"] = "false";
    }

    return list;
}

int TcpClass::ApplyEdit( ConfigurableItemList *list, Entity *entity)
{
    DEBUG(1,"Saving Data\n");
    Tcp *cp =static_cast<Tcp *>(entity);

    ConfigNode *ent=static_cast<Tcp*>(entity)->NODE();

    cp->hostname = (*list)["hostname"];

    cp->SetupHost();

    *ent->NODE("host")->PROPERTY("address")=
	(*list)["hostname"];
    *ent->NODE("host")->PROPERTY("port")=
	(*list)["port"];


    if ((*list)["muteflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("mute")=
	    (*list)["mute"];
    } else {
	try {
	    ent->NODE("option")->DeletePROPERTY("mute");
	} catch(...) {}
    }

    if ((*list)["warnflag"] == "true" ) {
	    *ent->NewNODE_IfNotFound("option")->NewPROPERTY("warn")=
		(*list)["warn"];
    } else {
	try {
	    ent->NODE("option")->DeletePROPERTY("warn");
	} catch(...) {}
    }
    if ((*list)["pflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("time")->NewPROPERTY("poll")=
	    (*list)["pvalue"];
    } else {
	try {
	    ent->NODE("time")->DeletePROPERTY("poll");
	} catch(...) {}
    }

    ent->DeleteNODE_IfEmpty("option");
    DEBUG(5,"This entity global setup\n");
    cp->Setup(ent);

    return 1;

}


int TcpClass::Save(ConfigurableItemList &list)
{

    DEBUG(5,"Saving data HERE\n");

    ConfigNode *ent = Config::NewNODE("entity");

    *ent->NewPROPERTY("type")= TcpName;
    *ent->NewPROPERTY("name")= list["name"];

    *ent->NewNODE("host")->NewPROPERTY("address")=
	list["hostname"];

    *ent->NODE("host")->NewPROPERTY("port")=
	list["port"];

    if (list["pflag"] == "true") {
	*ent->NewNODE_IfNotFound("time")->NewPROPERTY("poll") =
	    strint(list["pvalue"]);
    }
    
    if (list["muteflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("mute")=
	    list["mute"];
    }

    if (list["warnflag"] == "true" ) {
	*ent->NewNODE_IfNotFound("option")->NewPROPERTY("warn")=
	    list["warn"];
    }
    try {
	Config::GetApp()->Add( *Config::CallClassHandler( TcpName,
							  ent, *Config::GetVisual()));
    } catch (...) {

	delete(ent);
	return 0;
    }

    return 1;
}
