/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __TCP_H__
#define __TCP_H__

#include "config.h"
#include "Entity.h"
#include "TimeVal.h"
#include "Config.h"
#include "Visual.h"
#include "Led.h"
#include "ConfigItem.h"
#include <sys/types.h>
#include <netinet/in.h>

#ifdef HAVE_GTK
namespace Gtk {
    class SpinButton;
    class Box;
}
#endif

typedef enum {
	TCP_OK,
        TCP_WARN,
	TCP_FAIL,
        TCP_OFF
} TcpStateType;


const dstring TcpName("tcp");
const dstring TcpVisualName("TCP");

class TcpClass: public Configurable
{
public:
    friend class Tcp;
    TcpClass();
    virtual Entity *CreateEntity(ConfigNode*, VisualInterface&);
    virtual const dstring &Name() const { return TcpName; }
    virtual const dstring &VisualName() const { return TcpVisualName; }
    virtual void Edit(const dstring&);
    virtual void AddEntity();
    virtual void Rehash();
    static void Init();
    static int StaticFDHandler(int fd, FDTYPE type, void *data);
    void DoPoll();

    virtual ConfigurableItemList *Edit(Entity *);
    virtual ConfigurableItemList *Config();
    virtual ConfigurableItemList *Add();

    virtual int ApplyEdit( ConfigurableItemList *, Entity *);
    virtual int ApplyConfig( ConfigurableItemList *);
    virtual int ApplyAdd( ConfigurableItemList *);

    virtual int Save(ConfigurableItemList &list);
    void SaveData();
#if 0 // def HAVE_GTK
    Gtk::Box * NotebookPage();


    static void on_TcpSpinYellow_changed();
    static void on_TcpSpinRed_changed();
    static void on_TcpSpinPingInterval_changed();
    static void on_Tcp_destroy();
    static class Gtk::SpinButton *TcpSpinYellow;
    static class Gtk::SpinButton *TcpSpinRed;
    static class Gtk::SpinButton *TcpSpinPingInterval;
    static int cyellow,cred,ctime;
    
#endif

    // static int AllocRawSocket();
protected: /*
    static int fdsocket;
    static int rawsocket;
    static int timeoutid;
    */
    static list<Tcp*> TcpItems;

    static struct protoent *proto;
    /*
public:
    static int ident;
    static char outbuf[8+DATALEN];
    static char inbuf[MAXPACKET];
    */
};


#if 0 //def HAVE_GTK

#include <gtk--/window.h>

namespace Gtk {
    class CheckButton;
    class SpinButton;
    class Entry;
};

class TcpEntityWindow: public Gtk::Window
{
public:
    TcpEntityWindow(Tcp *p = 0);

    void on_APECheckYellow_clicked(const gchar *user_data);
    void on_APECheckRed_clicked(const gchar *user_data);
    void on_APECheckMuteFlag_clicked(const gchar *user_data);
    void on_APECheckWarnFlag_clicked(const gchar *user_data);
    void on_APEOKButton_clicked();
    void on_APECancelButton_clicked();
    gint on_delete_event(GdkEventAny*);
    int Save();
    Gtk::CheckButton *APECheckStartMuted,
	*APECheckDontWarn, *APECheckMuteFlag,
    *APECheckWarnFlag,
    *APECheckYellow,
    *APECheckRed;
    Gtk::SpinButton *APEYellowSpin,
	*APERedSpin;
    Gtk::Entry *APEHostnameEntry, *APEVisualnameEntry;
    Tcp *cp;
};

#endif


class Tcp: public Entity, public SigC::Object
{
public:

    Tcp(const char *hostname, int port, const char *visualname,VisualInterface&,
	   Led *l, TcpClass *);
    ~Tcp();
    
    int FDChanged(int fd,  int type);
    void FDHandler(int fd);

    virtual const dstring ClassName() { return dstring("tcp"); }
    void Poll();
    int Match(struct sockaddr_in &s) {
        return s.sin_addr.s_addr == sock.sin_addr.s_addr;
    }
    int Match(struct in_addr addr) {
        return addr.s_addr == sock.sin_addr.s_addr;
    }
    
    void GotConnect();
    //void GotUnreach(struct sockaddr_in &host);
  //  TimeVal Delay() { return lastping - lastreply; }
    void Setup(ConfigNode *);
    Entity* CreateEntity(ConfigNode *,VisualInterface&);
    void SetColor(LedColor c);
    void SetFlash(int c);
    
    friend class TcpClass;
    friend class TcpEntityWindow;
    ConfigNode *NODE() { return xmlnode; }
    //virtual void LeftClick();
    /*
    virtual void RightClick();
    virtual void MiddleClick();
    */
    void SetState(TcpStateType state);
    void SetState();
    int Port() {
        DEBUG(9,"Port id %d\n", port);
	return port;
    }
    //virtual ConfigurableItemList * Edit();
    
protected:
    int SetupHost();
    struct sockaddr_in sock;
    int fdsocket;
    int watchfdid;
    int port;
    int timeoutid;
    //int unreach;
    //int sequence;
//    TimeVal lastping;
 //   TimeVal lastreply;
    //static TimeVal def_yellow;
    //static TimeVal def_red;
    //TimeVal yellow,red;
    int warn, muted;
    TcpStateType currentstate;
    ConfigNode *xmlnode;
    int fail;
    /*
#ifdef HAVE_GTK
    TcpEntityWindow *PW;
    #endif
    */
};



#endif
