/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __GCMAIN_H__
#define __GCMAIN_H__
#include <string>
#include <gtk--/window.h>
#include <gtk--/box.h>
#include "ConfigItem.h"
#include <map>
/*
#include <gtk--/menubar.h>
#include <gtk--/entry.h>
#include <gtk--/checkbutton.h>
*/
namespace Gtk
{
    class SpinButton;
    class MenuBar;
    class Entry;
    class CheckButton;
    class CList;
    class Combo;
    class Button;
    class HBox;
    class VBox;
}

class App;

typedef enum {
    OK,
    OKCANCEL,
    YESNO
} DialogType;


class Entity;
class Configurable;

struct WindowData
{
    Gtk::Window *w;

    Gtk::Box *b;
    Gtk::Button *APEOKButton;
    Gtk::Button *APECancelButton;

    Gtk::HBox *APEHBoxButtons;
    Gtk::VBox *APEVBoxMain;
    Entity *entity;

    ConfigurableItemList *list;
    Configurable *eclass;
};



class ConfigWindow : public Gtk::Window
{   

public:
    ConfigWindow();
    ~ConfigWindow();
    void SetCloseNotify(const SigC::Slot1<void, void*> &, void *);
    static void CreateDialog(const string &, DialogType);
private:
    void on_save1_activate();
    void on_quit1_activate();
    void on_CommonOKText_changed();
    void on_CommonSelectText_changed();
    void on_CommonErrorText_changed();
    void on_CommonCheckRun_toggled(const gchar *user_data);
    void on_CommonTextEvent_changed();
    void on_RespawnTime_changed();
    void on_EntityList_select_row(gint row, gint column, GdkEvent *event);
    void on_EntityList_unselect_row(gint row, gint column, GdkEvent *event);
    void on_add_button_clicked(const gchar *user_data);
    void on_EntityDeleteButton_clicked(const gchar *user_data);
    void on_EntityEditButton_clicked(const gchar *user_data);
    void on_PingerSpinYellow_changed();
    void on_PingerSpinRed_changed();
    gint on_ConfigWindow_focus_in_event(GdkEventFocus *ev);
    gint on_ConfigWindow_delete_event(GdkEventAny *);
    void on_ConfigWindow_show();
    void on_ConfigWindow_close();
    void on_okButton_clicked();
    void on_cancelButton_clicked( );

    void on_APECancelButton_clicked( WindowData *);
    void on_APEOKButton_clicked( WindowData *);
    void on_APEOKButton_clicked_ADD(WindowData *data);
    void on_APEWindow_destroy(WindowData *data);

    WindowData *createWindow( ConfigurableItemList *list, Entity *, Configurable *,
			      const SigC::Slot1<void,WindowData*> &);

    void Entry_Changed( Gtk::Entry *entry, CItemString *);
    void Spin_Changed( Gtk::SpinButton *entry, CItemInteger *);
    void Check_Changed( Gtk::CheckButton *entry, CItemBoolean *);
    void Dependable_Toggle( Gtk::CheckButton *, pair<Gtk::Widget *, CItemBoolean *> *);
    void Dependable_Destroy( pair<Gtk::Widget *, CItemBoolean *> *);

    void AddEntity( Configurable * );
    void ChooseClass( );
    void on_ChooseClassCancel_clicked(Gtk::Widget *);
    void on_ChooseClassOK_clicked( Gtk::Combo *, Gtk::Widget *);

    class Gtk::MenuBar *menubar1;
    class Gtk::Entry *CommonOKText;
    class Gtk::Entry *CommonSelectText;
    class Gtk::Entry *CommonErrorText;
    class Gtk::Entry *CommonTextEvent;
    class Gtk::CheckButton *CommonCheckRun;
    class Gtk::SpinButton *RespawnTime;
    class Gtk::CList *EntityList;
    /*
     void (*close_notify)(void*);
     */
    void *cndata;

    SigC::Slot1<void,void*> close_notify;

    int selrow;

    void SaveData();
    void ReloadEntities();

    Gtk::Box * build_from_citemlist(const ConfigurableItemList *list);
    void parse_single( ConfigurableItem *item , Gtk::Box *container);

    map<Configurable *, ConfigurableItemList *> items;


    void message(const string &);
    void messageEnd(Gtk::Window *w);
};





#endif
