/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __CONFIGITEM_H__
#define __CONFIGITEM_H__

#include <list>
#include <string>

class ConfigurableItemList;


int strint(const string &);
string intstr(int);

class ConfigurableItem
{
public:
    ConfigurableItem(const string &_id, const string &name);

    virtual string & value() { return _value; }
    virtual void set_value(const string &s) { _value=s; }
    virtual const string get_type() = 0;
    const string &id() { return _id ; }
    const string &name() { return _name; }
    virtual bool isDependable() { return false; }
    virtual ConfigurableItem *findID(const string &i) {
	if (i==_id) return this;
	return NULL;
    }

protected:
//    virtual bool set_active() = 0;
    string _value;
    string _id,_name;
};

class CItemString: public ConfigurableItem
{
public:
    CItemString(const string &id, const string &name);
    virtual const string get_type() { return "string"; }
protected:
    string *def;
}
;
class CItemInteger: public ConfigurableItem
{
public:
    CItemInteger(const string &id, const string &name, int _min, int _max,const string &unit = "");
    virtual const string get_type() { return "integer"; }

    int minimum() { return min; }
    int maximum() { return max; }
    const string &units() { return _units; }
protected:
    int min,max;
    string _units;
};

class CItemGroup: public ConfigurableItem, public list<ConfigurableItem*>
{
public:
    CItemGroup(const string &name);
    virtual const string get_type() { return "group"; }
    virtual ~CItemGroup();
    virtual ConfigurableItem *findID(const string &i);
    CItemGroup &operator+=(ConfigurableItem *);
};


class CItemBoolean: public ConfigurableItem
{
public:
    CItemBoolean(const string &id, const string &name);
    virtual const string get_type() { return "boolean"; }
};

template <class T>
class CItemDependable: public T
{
public:
    CItemDependable(T *c, CItemBoolean *b): T(*c), bl(b) {
    };
    const string get_type() { return T::get_type(); }
    virtual bool isDependable() { return true; }
    CItemBoolean *getDependable() { return bl; }
    virtual ConfigurableItem *findID(const string &id)
    {
	if (bl->findID(id))
	    return bl->findID(id);
	return ConfigurableItem::findID(id);
    }
    virtual ~CItemDependable() {
    delete ( bl );
    }


protected:
    CItemBoolean *bl;
};


class ConfigurableItemList: public list<ConfigurableItem *>
{
public:
    ConfigurableItemList(ConfigurableItem *);
    ConfigurableItemList();
    ConfigurableItemList &operator+(ConfigurableItemList *);

    ConfigurableItemList &operator+(ConfigurableItem *);
    ConfigurableItemList &operator+=(ConfigurableItem *);

    virtual ~ConfigurableItemList();
    class not_found {} ;
    string &operator[](const string &id);
    virtual ConfigurableItem *findID(const string &i);
};

#endif
