/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __TIMEVAL_H__
#define __TIMEVAL_H__

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

/* Timeval extension */
struct TimeVal: public timeval
{
public:
    TimeVal(long usec) { tv_sec=0; tv_usec=usec; };
    TimeVal(long sec,long usec) { tv_sec=sec; tv_usec=usec; };
    TimeVal() { tv_sec=0; tv_usec=0; }
    TimeVal& operator=(const TimeVal&t) { tv_sec=t.tv_sec;
	tv_usec=t.tv_usec; return *this; }
    TimeVal& operator+=(const TimeVal&t) { tv_sec+=t.tv_sec;
	tv_usec+=t.tv_usec; if (tv_usec > 1000000L) { tv_sec++;
	    tv_usec-=1000000L; }
	return *this; };


    TimeVal& operator-=(const TimeVal&t) { tv_sec-=t.tv_sec;
	tv_usec-=t.tv_usec; if (tv_usec < 0) { tv_sec--;
	    tv_usec+=1000000L; }
	return *this; };


    TimeVal& operator+=(const long t) { tv_usec+=t;
	if (tv_usec > 1000000L) { tv_sec++;
	    tv_usec -= 1000000L; }
    return *this; };


    bool operator==(const TimeVal&t) const {
	return (tv_sec==t.tv_sec && tv_usec==t.tv_usec); }

    bool operator>(const TimeVal&t) const {
	return ((tv_sec==t.tv_sec && tv_usec>t.tv_usec)||
		tv_sec>t.tv_sec); }

    bool operator<(const TimeVal&t) const {
	return ((tv_sec==t.tv_sec && tv_usec<t.tv_usec)||
		tv_sec<t.tv_sec); }
};

TimeVal operator-(const TimeVal&a,const TimeVal&b);


class ClockTime: public TimeVal
{
public:
    ClockTime() { gettimeofday(this,0); }
    ClockTime(long seconds, long milisseconds)
    {
	gettimeofday(this,0);
        (*this)+=TimeVal(seconds,milisseconds);
    }
};

#endif
