/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    */

#ifndef __XMLCONFIG_H__
#define __XMLCONFIG_H__



#include <expat.h>
#include <hash_map>
#include <list>
#include <iostream.h>
#include "DString.h"
#include "Entity.h"
#include "Visual.h"
#include "Error.h"

#ifndef HAVE_LIBXMLPP

class M_XMLItem;

class M_XMLItemList: public list<M_XMLItem*>
{
public:
    M_XMLItem &operator[](const dstring&) ;
    M_XMLItem &Add(const dstring &n);
};

typedef M_XMLItemList::const_iterator M_XMLConstIterator;

class M_XMLProperty
{
public:
    M_XMLProperty(const dstring &n, const dstring &v) : name_(n), value_(v){};
    const dstring & name() const { return name_; }
    dstring & value() { return value_; }
    const dstring & value(const dstring&v) { return value_=v; }
private:
    dstring name_,value_;
};

ostream &operator<<(ostream &stream, const M_XMLProperty &property);


class M_XMLPropertyList: public list<M_XMLProperty*>
{
public:
    dstring & operator[] (const dstring &name) ;
    M_XMLProperty &operator() (const dstring &name) const {
	const_iterator it;
	for (it=begin(); it!=end(); it++) {
	    if (name == (**it).name())
		return (**it);
	}
        throw Error("No such property %s\n", name.Char());
    }
};

class M_XMLItem
{
public:
    M_XMLItem(const dstring &n, int i=0): name_(n), parent(0), internal(i)
    {
	if (internal) fprintf(stderr,"%s is internal\n",n.Char());
    };
    M_XMLItem(const char *n, const char **attrs, int i=0);
    const dstring & name() const { return name_; }

    // Child Lookup
    M_XMLItem & operator[] (const dstring &name) ;

    // Property Lookup
    dstring & operator() (const dstring &name)  {
        return properties[name];
    }

    M_XMLItem *AddChild(M_XMLItem *child) {
	Childs.insert(Childs.end(), child);
        return child;
    }
    void RemoveChild(M_XMLItem *child);

    const M_XMLItem *Child(const dstring &name);

    M_XMLConstIterator FirstChild() const { return Childs.begin(); }
    M_XMLConstIterator LastChild() const { return Childs.end(); }

    M_XMLProperty &AddProperty(const dstring&,const dstring&);
    void AddItem(M_XMLItem*item);

    const M_XMLItemList &childs() const { return Childs; }
    M_XMLItem &Add(const dstring &name) {
	/* Find first */
	try {
            return (M_XMLItem&)((*this)[name]);
	} catch (...) {
	    M_XMLItem *it =new M_XMLItem(name,1);
            it->parent = this;
	    return *AddChild(it);
	}
    }
    void AddProperties(const char **);

    dstring &Prop(const dstring &n) {
	try {
	    return (dstring&)
		(*this)(n);
	} catch (...) {
	    return (dstring&)AddProperty(n,"").value();
	}
    }

    M_XMLPropertyList::const_iterator FirstProperty() const { return properties.begin(); }
    M_XMLPropertyList::const_iterator LastProperty() const { return properties.end(); }
    int IsInternal() { return internal; }

private:
    dstring name_;
    M_XMLItemList Childs;
    M_XMLPropertyList properties;
public:
    M_XMLItem *parent;
    int internal;
};

class M_XMLParser
{
public:
    M_XMLParser(list<M_XMLItem*> &Root);
    int Parse(const char *file) const ;
    int ParseBuffer(const char *) const ;
    static void HandlerBegin(void *, const M_XML_Char *name,
		      const M_XML_Char **attrs);

    static void HandlerEnd(void *, const M_XML_Char *name);
protected:
    M_XML_Parser parser;
    static M_XMLItem *current;
    static list<M_XMLItem*> *RootItems;

};

#else

#include <xml++.h>

class XMLItem: public XMLNode
{
public:
    XMLItem(const XMLNode&n):
	XMLNode(n) {};
    dstring *item(const dstring &location);
};

typedef XMLNodeList XMLItemList;

class XMLParserCallbacks {
public:
  virtual void DocumentStart(void);
  virtual void DocumentEnd(void);
  virtual void ElementStart(const dstring &n, const XMLPropertyHash &p);
  virtual void ElementEnd(const dstring &n);
  virtual void Characters(const dstring &s);
  virtual void Comment(const dstring &s);
  virtual void Warning(const dstring &s);
  virtual void Error(const dstring &s);
  virtual void FatalError(const dstring &s);
};

class M_XMLParser: public XMLParser<XMLParserCallbacks>
{
public:
    M_XMLParser(XMLTree &Root);
    int Parse(const char *file) const ;
    int ParseBuffer(const char *) const ;
    static void HandlerBegin(void *, const XML_Char *name,
		      const XML_Char **attrs);

    static void HandlerEnd(void *, const XML_Char *name);
protected:
    xmlSAXHandler saxh;
    //XMLParserCallbacks callbacks;

    XML_Parser parser;
    static XMLNode *current;
    static XMLNodeList *RootItems;
};


#endif

/*
class Configurable;
*/
class ClassTypeHandler
{
public:
    typedef Entity* (*Handler)(XMLItem&,VisualInterface&) ;
};

class Configurable:public pair<const char*,ClassTypeHandler::Handler>
{
public:
    Configurable(const char *c,ClassTypeHandler::Handler h):
	pair<const char*,ClassTypeHandler::Handler>(c,h)
    {}
};


/*
class ConfigRegistry
{
public:
    ConfigRegistry();
    static XMLItem *FindRootItem(const char*name);
    static void RegisterClass(Configurable *c) {
        Classes.insert(Classes.end(),c);
    }

    typedef XMLNodeList::const_iterator RootIterator;
    static ClassTypeHandler::Handler GetClassHandler(const char *name);

    static XMLItemList RootItems;
    static XMLItem Defaults;

    static list<Configurable *> Classes;
    const M_XMLParser &Parser() { return parser; }
    static int Load(const char *file);
    static int Save();

    static XMLProperty *item(const dstring &location);

protected:
    static M_XMLParser parser;
    static dstring filename;
};
*/

namespace ConfigRegistry
{
    typedef list<Configurable*> ClassList;
    ClassTypeHandler::Handler GetClassHandler(const dstring &);
    void RegisterClass(Configurable *c);
    XMLProperty *item(const dstring &location);

    ClassList Classes;
    XMLTree RootItems;
    XMLNode Defaults=XMLNode("monitor");

    M_XMLParser parser(RootItems);
    int Load(const char *file);
}


/*
class ClassType
{
public:
    ClassType(const char *c, ClassTypeHandler::Handler h )
    {
        fprintf(stderr, "ClassType new %s\n",c);
	ConfigRegistry::RegisterClass( new Configurable(c,h) );
    }
};
               */
/* Stream misc. operations */

ostream &operator<<(ostream &stream, const XMLProperty &property);
ostream &operator<<(ostream &stream, const XMLItem &item);
//ostream &operator<<(ostream &stream, const ConfigRegistry &reg);
ostream &operator<<(ostream &stream, const XMLItemList &reg);

#endif
