/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "GTKApp.h"
#include <stdio.h>
#include "Entity.h"
#include "Config.h"
#include "Error.h"
#include <sys/signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include "debug.h"
#include <gtk--/fixed.h>
#include <dlfcn.h>
#include "../gtkconfig/GCMain.h"

void *CreateVisual(int argc, char **argv)
{
    DEBUG(5,"Creating GTKApp\n");
    return new GTKApp(argc,argv);
}


void GTKApp::chld_signal(int sig)
{
    while (waitpid(0, (int *) (NULL), (WNOHANG | WUNTRACED)) > 0);
}

void GTKApp::Configure()
{
    Config::Iterator it;
    DEBUG(4,"Configuring entities\n");
    for (it = Config::Begin();
	 it != Config::End();
	 it++) {
	DEBUG(5, "Name: %s\n",(*it)->name().Char());
	if ((*it)->name() == "entity") {
	    Add ( *Config::CallClassHandler((*it)->PROPERTY("type")->VALUE(),
					    (*it), *this) );
	}
    }
}

void bdrawrect(unsigned char *data, int w, int h, int x, int y, int width,
	      int height)
{
    int i,k;
    register int linestart =((w/8)+(w%8!=0));

    for (i=y; i<(y+height); i++)
    {
	for (k=x; k<(width+x); k++)
	    data[ (i*linestart) + (k/8)] |= 1<<(k%8);
    }
}

size_t bitmapsize(int w, int h)
{
    return ((w/8)+(w%8!=0)) * h;
}

GTKApp::GTKApp(int argc, char **argv):
GTKInterface(argc,argv),
CreateConfigWindow(0)
{
    unsigned char *data;
    DEBUG(5,"Creating app and loading config\n");
    void *handler = dlopen(VISUAL_LIBDIR "/libmonitorit_config_gtk.so",
			   RTLD_NOW|RTLD_GLOBAL);
    if (handler==0) {
	DEBUG(1, "Cannot open: %s\n", dlerror());
	throw Error("Error");
    }

    CreateConfigWindow = (Configurator)dlsym(handler,"newConfigWindow__Fv");
    if ((void*)CreateConfigWindow==0) {
	DEBUG(1, "Cannot open: %s\n", dlerror());
	throw Error("Error");
    }

    Config::SetVisual(this);
    Config::SetApp(this);
    Open(argc,argv,0,0,0);

    cx = 0;
    cy = 0;

    lc = new GTKLedConfig(*this, 0,0,0,0,0);

    Gtk::Fixed *fixed = manage(new Gtk::Fixed());


    data = (unsigned char*)g_malloc( bitmapsize(211,135));
    memset(data, 0x0, bitmapsize(211,135));

    // Text Area

    bdrawrect(data, 211, 135, 10,24,190,17);

    // Drawing Area

    bdrawrect(data, 211, 135, 10,66,190,59);

    Gdk_Window win = window.get_window();
    Gdk_Bitmap bitmap(win , (char*)data, 211,135);
    Gdk_Color black;
    GetXColor("0x000000",black);
    style = fixed->get_style()->create();
    style->set_bg(GTK_STATE_NORMAL, black);
    fixed->set_style(*style);
    fixed->shape_combine_mask(bitmap,0,0);

    led_area = new GTKFixed();
    led_area->set_style(*style);
    led_area->set_usize(190,59);
    led_area->set_events(GDK_ENTER_NOTIFY_MASK|GDK_LEAVE_NOTIFY_MASK|
			 GDK_BUTTON_PRESS_MASK);
    fixed->put(*led_area, 10, 66);
    led_area->show();

    text_area = manage(new GTKText(*this));
    text_area->set_style(*style);
    text_area->set_usize(190,17);
    fixed->put(*text_area, 10, 24);

    text_area->show();
    fixed->show();

    Configure();

    window.add(*fixed);

    

    led_area->enter_notify_event.connect( SigC::slot (this, &GTKApp::EnterLedArea) );
    led_area->leave_notify_event.connect( SigC::slot (this, &GTKApp::LeaveLedArea) );
    led_area->button_press_event.connect( SigC::slot (this, &GTKApp::LedAreaClick) );


    const char *font;
    try {
	font = Config::NODE("text")->PROPERTY("font")->VALUE().Char();
        text_area->SetFont(font);
    } catch (...) {
    }
    

    const char *color;
    try {
	color = Config::NODE("text")->PROPERTY("color")->VALUE().Char();
    } catch (...) {
	color="cyan";
    }
    text_area->SetColor(color);

    text_area->SetText(Config::NODE("text")->PROPERTY("oktext")->VALUE().Char());

    failures = 0;

    window.show();
    DEBUG(5,"Done creating GTKApp\n");
}

int GTKApp::EnterLedArea(GdkEventCrossing*)
{
//    DEBUG(5,"Entering Led Area\n");
    text_area->SetTemporaryTextNoScroll(Config::NODE("text")->PROPERTY("selecttext")->VALUE().Char());
    return TRUE;
}

int GTKApp::LeaveLedArea(GdkEventCrossing*)
{
 //   DEBUG(5,"Leaving Led Area\n");
    text_area->CancelTemporary();
    return TRUE;
}

int GTKApp::LedAreaClick(GdkEventButton*)
{
    DEBUG(5,"Led Area Click\n");
#ifdef HAVE_GTK
    RunConfig();
#endif
    return TRUE;
}

int GTKApp::EnterLed(GdkEventCrossing*,Entity *e)
{
    DEBUG(5,"Entering Led\n");
    text_area->SetTemporaryTextNoScroll(e->VisualName().Char());
    return TRUE;
}

int GTKApp::LeaveLed(GdkEventCrossing*,Entity *e)
{
    DEBUG(5,"Leaving Led\n");
    text_area->CancelTemporary();
    return TRUE;
}

int GTKApp::ButtonPressLed(GdkEventButton*event,Entity *e)
{
    DEBUG(5,"Led Press\n");
    switch (event->type) {
    case GDK_BUTTON_PRESS:
	switch (event->button) {
	case 1:
	    e->LeftClick();
	    break;
	case 2:
	    e->RightClick();
	    break;
	case 3:
            
	    e->MiddleClick();
	    Gtk::Main::quit();
	    break;
	}
    }
    return TRUE;
}
int GTKApp::ButtonReleaseLed(GdkEventButton*event,Entity *e)
{
    DEBUG(5,"Led Press\n");
    switch (event->button) {
    }
    return TRUE;
}



void GTKApp::IFail(Entity*g)
{
    dstring error;

    error = Config::NODE("text")->PROPERTY("errortext")->VALUE();
    if (!failures)
    {
	ptime = RequestPeriodicTimeout(Config::NODE("fail")->
				       PROPERTY("respawn")->
				       VALUE().Integer(), 0,
				       slot(this, &App::periodic) );
        periodic();
    }
    failures++;

    FailItems.insert(FailItems.end(), g);

    list<Entity*>::const_iterator it;
    for (it=FailItems.begin();
	 it!=FailItems.end();
	 it++)       {
	error+=" <";
	error+=(*it)->VisualName();
	error+=">";
    }

    text_area->SetText(error.Char());
}


void GTKApp::IDoNotFail(Entity*g)
{
    failures--;
    if (failures == 0) {
	text_area->SetText(Config::NODE("text")->PROPERTY("oktext")->VALUE().Char());
        CancelTimeout(ptime);
    }

    list<Entity*>::iterator it;
    for (it=FailItems.begin();
	 it!=FailItems.end();
	 it++){
	if (g == (*it)) {
	    FailItems.erase(it);
	    break;
	}
    }

}

void GTKApp::Add(Entity &entity)
{
    DEBUG(5,"Adding Entity at %d/%d\n",cx,cy);

    GTKLed *l;

    l= dynamic_cast<GTKLed*> (&entity.GetLed());
    l->set_events(GDK_ENTER_NOTIFY_MASK|
		  GDK_LEAVE_NOTIFY_MASK|
		  GDK_BUTTON_PRESS_MASK);

    led_area->put( *l ,cx ,cy);
    
    l->show();


    cx+=l->Width()+1;
    if (cx+l->Width()+1 > led_area->Width()) {
	cy+=l->Height()+1;
	cx=0;
    }

    SigC::Slot1<int,GdkEventCrossing*> led_enter =
    bind(slot(this, &GTKApp::EnterLed), &entity);

    SigC::Slot1<int,GdkEventCrossing*> led_leave =
    bind(slot(this, &GTKApp::LeaveLed), &entity);


    l->enter_notify_event.connect(led_enter);
    l->leave_notify_event.connect(led_leave);
    l->button_press_event.connect(SigC::bind(SigC::slot(this,&GTKApp::ButtonPressLed),
					     &entity));

    Items.insert(Items.begin(), &entity);
}

void GTKApp::AttachConfigure(Gtk::Widget *(*ccw)())
{
//    CreateConfigWindow =  ccw;
}

void GTKApp::RunConfig()
{
    //visual->Configure();
    DEBUG(5,"Running configurator\n");
    if (!configuring) {
	MyConfigWindow = CreateConfigWindow();
	MyConfigWindow->SetCloseNotify(SigC::slot(this, &GTKApp::ConfigEnd), (void*)this);
	MyConfigWindow->show();
        configuring=1;
    }
    DEBUG(5,"Config running!\n");
}


void GTKApp::ConfigEnd(void *)
{
    DEBUG(5,"Config Ended");
//    delete(MyConfigWindow);
    configuring = 0;
}

GTKApp::~GTKApp()
{
  // This is managed  delete(lc);
}



