/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "GTKLed.h"
#include <stdio.h>
#include "Error.h"
#include <big_leds.xpm>
#include <gtk--/pixmap.h>
#include "GTKVisual.h"
#include "debug.h"

Gdk_Pixmap GTKLedConfig::all_leds = 0;
Gdk_Bitmap GTKLedConfig::leds_mask = 0;

int GTKLedConfig::width = 0;
int GTKLedConfig::height = 0;

int GTKLedConfig::xpm_x=0;
int GTKLedConfig::xpm_y=0;

//int GTKLedConfig::timeoutid = 0;

int GTKLed::initialized = 0;

GTKLed::GTKLed(GTKInterface &v):
    GTKWidget(v), flashing(0), color(LED_OFF), c1(LED_RED)
{
    if ( ! initialized) {

    }
   /*
    SetSize( GTKLedConfig::width,
    GTKLedConfig::height);
    */
    shape_combine_mask(GTKLedConfig::leds_mask, 0, 0);
    set_usize(GTKLedConfig::width,
	      GTKLedConfig::height);
    DEBUG(5,"Creating LED\n");
}
/*void GTKLed::Info()
{
    fprintf(stderr,"X:%d Y:%d\n",x,y);
}
*/
GTKLedConfig::GTKLedConfig(GTKInterface &visual,const char *filename,int sx,int sy, int w, int h)
{
    if (filename) {
	all_leds.create_from_xpm(visual.GetWindow().get_window(),
				 leds_mask,
				 Gdk_Color("white"),
				 filename);
	xpm_x = sx;
	xpm_y = sy;
	width=w;
	height=h;
    }
    else {
	all_leds.create_from_xpm_d(visual.GetWindow().get_window(),
				 leds_mask,
				 Gdk_Color("white"),
				 big_leds_xpm);

	xpm_x = 0;
	xpm_y = 0;
	width = 8;
        height = 8;
    }
}

int GTKLed::SetFlash(bool b)
{
    list<Led*>::iterator it;
    if (flashing==b)
	return 0;
    if (flashing) {
	/* Stop Flash */
	
	for (it=Flash.begin(); it!=Flash.end(); it++)
	{
	    if ((*it)==this) {
		Flash.erase(it);
		flashing = 0;
                Redraw();
		break;
	    }
	}
        if (Flash.empty()) {
	    visual->CancelTimeout(timeoutid);
	}
    }
    else {
	/* Start Flash */
	if (Flash.empty())
	{
	    timeoutid = visual->RequestPeriodicTimeout(0 , 500000L, SigC::slot(this,&GTKLed::DoFlash));
	}

	Flash.insert(Flash.begin(),this);
	flashing = 1;
        Redraw();
    }
    return 1;
}


void GTKLed::DoFlash()
{
    list<Led*>::iterator it;
    for (it=Flash.begin(); it!=Flash.end(); it++)
    {
	Led * p = (Led*)(*it);
	if (p->GetColor() == LED_OFF)
	    p->SetTrueColor();
	else
	    p->SetTempColor(LED_OFF);
	p->Redraw();
    }
}

void GTKLed::Redraw()
{     /*
    get_style()->get_fg_gc(get_state()).
	set_clip_rectangle(&p->area);
	*/
    /*Gdk_Pixmap pxmap;
    Gdk_Bitmap bitm;

    get(pxmap,bitm);

    pxmap.draw_pixmap(      	      get_style()->get_fg_gc(get_state()),
			      GTKLedConfig::all_leds,
			      GTKLedConfig::xpm_x + (Led::color * GTKLedConfig::width ),
			      GTKLedConfig::xpm_y,
			      0, 0,
			      GTKLedConfig::width,
			      GTKLedConfig::height
			      );
                              */
    draw(0);

}

void GTKLed::DrawAt(int xpos, int ypos)
{
    /*
    //fprintf(stderr,"GTKLed - Drawing at %d, %d, my size is %d,%d, my pos %d,%d\n",xpos,ypos,width,height,x,y);
    XSetClipOrigin(visual->GetDisplay(),
		   visual->GetDefaultGC(),
		   xpos,ypos);

    XSetClipMask(visual->GetDisplay(),
		 visual->GetDefaultGC(),
		 LedConfig::leds_mask);

    visual->DrawPixmap(LedConfig::all_leds,
		       LedConfig::xpm_x + (color*LedConfig::width),
		       LedConfig::xpm_y,
		       LedConfig::width,
		       LedConfig::height,
		       xpos, ypos);

    XSetClipOrigin(visual->GetDisplay(),
		   visual->GetDefaultGC(),
		   0, 0);
    XSetClipMask(visual->GetDisplay(),
		 visual->GetDefaultGC(),
		 None);
		 */
    realize_if_needed();
}

#include "debug.h"

gint GTKLed::expose_event_impl(GdkEventExpose *p)
{
    //    DEBUG(5,"Exposing with color %d\n",Led::color);
    
                                           /*
    get_style()->get_fg_gc(get_state()).
	set_clip_rectangle(&p->area);
                                             */
    //shape_combine_mask(GTKLedConfig::leds_mask, 0, 0);
    get_window().draw_pixmap( 
			      get_style()->get_fg_gc(get_state()),
			      GTKLedConfig::all_leds,
			      GTKLedConfig::xpm_x + (Led::color * GTKLedConfig::width ),
			      GTKLedConfig::xpm_y,
			      0, 0,
			      GTKLedConfig::width,
			      GTKLedConfig::height
			    );
    return Gtk::DrawingArea::expose_event_impl(p);
}

void GTKLed::IFail(Entity*e) { visual->IFail(e); }
void GTKLed::IDoNotFail(Entity*e) { visual->IDoNotFail(e); }

