/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "GTKText.h"
#include "GTKVisual.h"
//#include <stdio.h>

#include "debug.h"

GTKText::GTKText(GTKInterface&v):
    GTKWidget(v),
    Gtk::DrawingArea(),
    font("fixed")
{
    textistemp=0;
    xoffset = 0;
    xborder=2;
    yborder=2;
    render=0;
    visual->GetXColor("white", color);
    visual->GetXColor("black", bgcolor);
    visual->RequestPeriodicTimeout(0,20000L,  SigC::slot(this,&GTKText::AllScroll));
}

void GTKText::SetSize(int w, int h)
{
   set_usize(w,h);
}

void GTKText::SetColor(const char *c)
{
    visual->GetXColor(c,color);
}

void GTKText::SetBGColor(const char *c)
{
    visual->GetXColor(c,bgcolor);
}

void GTKText::SetFont(const char *f)
{
    font = Gdk_Font(f);
}

void GTKText::DrawText(const dstring &text)
{
    DEBUG(5,"Creating pixmap %d,%d\n",tw, Height() );

    render = Gdk_Pixmap(get_window(), tw, Height() );

    gc.create(get_window());
    gc.set_foreground(bgcolor);
    render.draw_rectangle(gc, TRUE,0,0, tw,
			   Height() );
    gc.set_foreground(color);
    render.draw_text(font,
		     gc,
		     textistemp ? (Width()/2)-(tw/2) : 0,
		     Height() - yborder - text_yasc,
		     text.Char(), text.size());

}

void GTKText::NextText()
{
    //int dir;
    text = nexttext;
    nexttext.erase();

    tw = font.string_width(text.Char());
    text_yasc = font.descent();

    DrawText(text);
}


void GTKText::SetText(const char *t)
{
    DEBUG(5,"Setting text to %s\n",t);
    nexttext = t;
    /* If first time */
    if (!text.size()) {
	NextText();
    }
}

void GTKText::Scroll()
{
    if (!textistemp) {
	xoffset++;
	if (xoffset > tw) {
	    xoffset=-Width();
	    // Change Text
	    if (nexttext.size()) {
		NextText();
	    }
	}
	else Redraw();
    } else Redraw();
}

void GTKText::AllScroll(void)
{
    Scroll();
}

int GTKText::expose_event_impl(GdkEventExpose*)
{
    static int wastemp = 0;
//     DEBUG(5,"Exposing text %s\n",t);
    /*

    visual->SetForeground(&bgcolor);

    XFillRectangle(visual->GetDisplay(),
	      renderarea,
	      visual->GetDefaultGC(),
	      0,
	      0,
	      width,
		   height);

    visual->SetForeground(&color);
    */
    /*
    Gdk_GC gc;
    gc.create(get_window());
    gc.set_foreground(color);

    get_window().clear_area(textistemp ? (Width()/2)-(tw/2) : -xoffset,
			    0,
			    tw,Height()
			   );
    get_window().draw_text(font,
			   gc,
			   textistemp ? (Width()/2)-(tw/2) : -xoffset,
			   Height() - yborder - text_yasc,
			   t, size);

			   */
    if (textistemp) {
	get_window().clear_area(0,0,Width(),Height()
			       );
	gc.set_foreground(color);
	get_window().draw_text(font,
			       gc,
			       (Width()/2)-(font.string_width(temptext.Char())/2),
			       Height() - yborder - text_yasc,
			   temptext.Char(),temptext.size());

        wastemp = 1;

    } else {
	if (wastemp) {
	    get_window().clear_area(0,0,Width(),Height());
	    wastemp = 0;
	    // Give some time
            return TRUE;
	}
	get_window().draw_pixmap(gc, render, 0, 0, -xoffset,
				 0); //Height() - yborder - text_yasc
    }
    return TRUE;
}

void GTKText::SetTemporaryTextNoScroll(const char *text)
{
    textistemp = 1;
    temptext = (char*)text;
    //DrawText(temptext);
}

void GTKText::CancelTemporary()
{
    textistemp = 0;
    temptext.erase();
    //DrawText(text);
}



//GTKText::~GTKText() {}
