/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/select.h>
#include <sys/time.h>
#include "GTKVisual.h"
#include "Setup.h"
#include "Error.h"
#include "skin.xpm"
#include "GTKLed.h"
#include <gtk--/pixmap.h>
#include <gtk--/drawingarea.h>
#include <gtk--/box.h>
#include <gtk--/button.h>
#include "debug.h"
#include <sigc++/convert.h>
int GTKInterface::timeoutid = 1;
list<GTKFD> GTKInterface::FDS;

int GTKInterface::OpenGTK(int argc, char **argv)
{
    const char *dname=0;
    int carg;

    for (carg=1; carg<argc; carg++) {
	if (strcmp(argv[carg],"-display")==0) {
	    dname=argv[carg+1];
            carg++;
	}
        if (strcmp(argv[carg],"-geometry")==0) {
	    geometry=argv[carg+1];
            carg++;
	}
    }

    geometry = 0;
    appname = argv[0];

    return 1;
}

int GTKInterface::SetupWindow(int width, int height)
{
    window.set_policy(FALSE, FALSE, FALSE);
    window.set_wmclass(appname, appname);
    window.set_default_size(width, height);
    window.set_usize(width, height);
    window.realize();
    window.get_window().set_decorations( GdkWMDecoration(0) );

    return 1;
}

int GTKInterface::CreateWindow()
{
    window.show();
    return 1;
}


int GTKInterface::CloseVisual()
{
    return 1;
}

int GTKInterface::GetXColor(const char *name, Gdk_Color &dest)
{
    dest.parse(name);
    window.get_colormap().alloc(dest);
    return 1;
}
void GTKInterface::Resize(int w, int h)
{
    width = w;
    height = h;
}

int GTKInterface::CheckEvent()
{
    return 1;

}

GTKInterface::GTKInterface(int argc ,char**argv):
Gtk::Main(argc,argv),
default_sleep_time(2,0L),
window(GTK_WINDOW_TOPLEVEL)
{
    manage(&window);
}

GTKInterface::~GTKInterface()
{
}

      /*
int  XpmFileScanHW(const char *file, int *width, int *height)
{
    int numcol, depth;
    FILE *f = fopen(file,"r");
    if (!f) return 0;

    fscanf(f, "%d %d %d %d", width, height, &numcol, &depth);
    fclose(f);
    return 1;
}
        */
int GTKInterface::SetShape(const char *xpmfile)
{
    /*
    //int h,w;
    Atom type;
    int format;
    unsigned long items, after;
    MwmHints *hints,newhints;
    newhints.flags = 0;
    if (xpmfile) {
	int x = XpmReadFileToPixmap (display,
				     window,
				     (char*)xpmfile,
				     &shape,
				     &shapemask,0);
	if (x<0)
	    return 0;
    } else {
	XpmCreatePixmapFromData(display, window,
				skin_xpm,
                                &shape, &shapemask, 0);
    }

    */
    DEBUG(5,"Entering SetShape\n");
    //    Gtk::Pixmap *P;

    
    if (!xpmfile)
	shape.create_from_xpm_d(window.get_window(),
				shapemask,
				Gdk_Color("white"),
				skin_xpm);
    else
	shape.create_from_xpm(window.get_window(),
			      shapemask,
			      Gdk_Color("white"),
			      xpmfile);


    //shape.create_from_xpm( *this , shapemask, Gdk_Color(0), xpmfile);
                                                         /*
							  XpmFileScanHW(xpmfile, &w, &h);                        */

    window.get_window().set_back_pixmap(shape, 0);

    window.shape_combine_mask( shapemask, 0, 0);
    window.SetSkin(shape, shapemask);

    /*

    window.get_style()->set_background(window.get_window(), GTK_STATE_NORMAL);

    Gdk_Pixmap s = window.get_style()->get_background(GTK_STATE_NORMAL);
    Gdk_GC gc;
    gc.create(window.get_window());

    s.draw_pixmap( gc, shape, 0,0,0,0);
    window.add(*a);
*/
    return 1;
}

int GTKInterface::Open(int argc, char **argv,int wwidth, int wheight)
{
    width = 211;
    height = 135;
    if (!OpenGTK(argc,argv))
        throw Error("Cannot Open Display");
    if (!SetupWindow(width, height))
        throw Error("Cannot Set Up Window");
    if (!CreateWindow())
	throw Error("Cannot Create Window");

    return 1;
}
int GTKInterface::Open(int argc, char **argv,int wwidth, int wheight,
		       const char *xpmfile)
{
//    int w,h;
    /*
    if (xpmfile) {
	if (!XpmFileScanHW(xpmfile,&w,&h))
	    throw Error("File not found: %s",xpmfile);
	width = w;
	height = h;
    } else {*/
	width = 211;
        height = 135;
/*    }*/
    if (!OpenGTK(argc,argv))
        throw Error("Cannot Open Display");
    if (!SetupWindow(width, height))
        throw Error("Cannot Set Up Window");
    if (!SetShape(xpmfile))
        throw Error("Cannot Set Window Shape");
    if (!CreateWindow())
	throw Error("Cannot Create Window");
    return 1;
}


int GTKInterface::WatchFD(int fd, FDTYPE itype, const SigC::Slot1<void, int> & cb )
{
    int type = 0;
    //=GDK_INPUT_READ;

    if (itype&FDREAD)
	type|=GDK_INPUT_READ;
    if (itype&FDWRITE)
	type|=GDK_INPUT_WRITE;
    if (itype&FDEXCP)
	type|=GDK_INPUT_EXCEPTION;
    
    DEBUG(5,"Requesting to watch File Descriptor %d type %d\n",fd,type);

    FDS.push_back(
		  GTKFD( fd, (GdkInputCondition)type, cb,
			 Gtk::Main::input.connect( bind ( slot (this, &GTKInterface::FDH),
							  cb ), fd, GdkInputCondition(itype)
						)
		       )
		 );
			    
    return 1;
}

void GTKInterface::FDH(int f, GdkInputCondition c, Slot1<void,int> h)
{
    DEBUG(5,"File Descriptor %d changed\n",f);
    h.call( f );
}


int GTKInterface::RunVisual()
{
    run();
    return 1;
}


void GTKInterface::FlushExposeWindow(int w)
{
}
int GTKInterface::RequestPeriodicTimeout(long seconds, long milisseconds,
					 const SigC::Slot0<void> &cb )
{
  //  DEBUG(5,"Requesting timeout\n");
    timeoutid++;
    SigC::Slot0<gint> callback =
	bind(slot(this,&GTKInterface::TimeoutH),timeoutid);

    Timeouts.insert(Timeouts.begin(),
		    GTKTimeout(Timeout::Persistent,
			       seconds, milisseconds,
			       cb ,timeoutid,
			       Gtk::Main::timeout.connect(callback, (milisseconds/1000L)+(seconds*1000L))));
    return timeoutid;
}

int GTKInterface::TimeoutH(int number)
{
//    DEBUG(5,"Timeout expired\n");
    list<GTKTimeout>::iterator it;
    for (it = Timeouts.begin(); it!=Timeouts.end(); it++)
    {
	if ((*it).Id() == number)
            (*it).Call();
    }
    return TRUE;
}

int GTKInterface::CancelTimeout(int id)
{
    list<GTKTimeout>::iterator it;
    for (it=Timeouts.begin(); it!=Timeouts.end(); it++)
    {
	if ((*it).Id() == id) {
            (*it).conn.disconnect();
	    Timeouts.erase(it);
	    break;

	}
    }
    return 1;
}

int GTKInterface::StopWatch(int fd)
{
    list<GTKFD>::iterator it;
    DEBUG(5,"Stopping watch for %d\n",fd);
    for (it=FDS.begin(); it!=FDS.end(); it++)
    {
	if ((*it).fd == fd)
	{
            DEBUG(5,"Stopping watch\n");
	    (*it).c.disconnect();
	    FDS.erase(it);
	    return 1;
	}
    }
    DEBUG(1,"while Stopping watch - not found!!!\n");
    return 0;
}

int GTKInterface::Exec(const string &command,
		       const vector<string> &args,
		       const ExecCallback &c)
{
    int pid;
    char const **argv;
    argv = new char const *[args.size()+1];
    for (int cnt = 0; cnt<args.size(); cnt++) {
	argv[cnt] = args[cnt].c_str();
    }
    argv[args.size()] = NULL;


    /* Install it first !!!!! */


    switch (pid=vfork()) {
    case 0:
	/* Child */
        execve(command.c_str(), (char*const*)argv, NULL);
	_exit(-1);
    case -1:
	delete (argv);
        return -1;
    default:
	execs[pid] = c;
	delete(argv);
        return pid;
    }
}

Led *GTKInterface::NewLed()
{
    DEBUG(5,"Creating new GTK Led\n");
    return new GTKLed(*this);
}

