/*
    MonitorIT - A network monitoring tool
    Copyright (C) 2000-2001 Alvaro Lopes <alvieboy@alvie.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __GTKVISUAL_H__
#define __GTKVISUAL_H__

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <list>
#include "Visual.h"
#include "Error.h"
#include <gtk--/window.h>
#include <gtk--/main.h>
#include <gtk--/style.h>
#include <sigc++/slot.h>
#include "GTKWindow.h"
#include <hash_map>

using namespace SigC;

class GTKFD {
public:
    GTKFD(int f,GdkInputCondition t,const VisualInterface::FDChangedHandler h, const Connection &conn) {
	fd=f;
	type=t;
	handler=h;
        c=conn;
    }
    int CallHandler(){
	handler.call(fd);
    }
    int fd;
    GdkInputCondition type;
    VisualInterface::FDChangedHandler handler;
    void *data;
    Connection c;
};

class GTKTimeout: public Timeout
{
public:
    GTKTimeout(int t, int seconds, int milisseconds, SigC::Slot0<void> cb, int id,Connection c) :
	Timeout(t,seconds,milisseconds,cb,id),
        conn(c)
	{}
	Connection conn;
};



/*
typedef struct {
    unsigned long flags;
    unsigned long functions;
    unsigned long decorations;
    long input_mode;
    unsigned long status;
} MwmHints;

#define MWM_HINTS_DECORATIONS   (1L << 1)

#define MWM_DECOR_ALL           (1L << 0)
#define MWM_DECOR_BORDER        (1L << 1)
#define MWM_DECOR_RESIZEH       (1L << 2)
#define MWM_DECOR_TITLE         (1L << 3)
#define MWM_DECOR_MENU          (1L << 4)
#define MWM_DECOR_MINIMIZE      (1L << 5)
#define MWM_DECOR_MAXIMIZE      (1L << 6)

*/
class GTKInterface: public VisualInterface,
public Gtk::Main
{
public:
//    typedef list<GTKFD>::iterator FDit;

    GTKInterface();
    GTKInterface(int,char**);
    ~GTKInterface();
    int Open(int argc, char **argv,int,int);
    int Open(int argc, char **argv,int,int,const char *);
    int CloseVisual();
    int RunVisual();
    //    int WatchFD(int,int,VisualInterface::FDChangedHandler handler,void*);
    int WatchFD(int fd, FDTYPE fdtype, const SigC::Slot1<void, int> &);
    int StopWatch(int fd);
    Gtk::Window &GetWindow() { return window; }
    Gdk_GC GetDefaultGC() { return window.get_style()->get_white_gc(); }
    /*void DrawPixmap(Gdk_Pixmap p, int x, int y,
		    int sx, int sy, int dx, int dy);*/
    const char *Type() { return "GTK"; }
    void DrawRectangle(int x, int y, int w, int h);
    void DrawFilledRectangle(int x, int y, int w, int h);
    void SetFill(int);
    Gdk_Font GetFont(const char *font);
    //void FreeFont(Font);
    void DrawText(int x, int y, Gdk_Font, char *text);
    void SetForeground(Gdk_Color *);
    void SetBackground(Gdk_Color *);
    int GetXColor(const char *name, Gdk_Color &dest);
    int RequestPeriodicTimeout(long seconds, long milisseconds,
			       const SigC::Slot0<void> & );
    int CancelTimeout(int);
//    int GetDepth() { return defaultdepth; }
    int Exec(const string &command, const vector<string> &, const ExecCallback &);
protected:
    /*
     Methods
     */

    //virtual void HandleEvent(XEvent &event);
    
    void ShowVMM();
    TimeVal ComputeTimeout();
    int OpenGTK(int argc, char **argv);
    int SetupWindow(int width, int height);
    int CreateWindow();
    //Gdk_Color &GetColor(const char*);
    int CheckEvent();
    void Resize(int,int);
    void FlushExposeWindow(int);
    int SetShape(const char *xpmfile);

    Led *NewLed();

    static list<GTKFD> FDS;

    list<GTKTimeout> Timeouts;

    TimeVal default_sleep_time;

    void FDH(int, GdkInputCondition, SigC::Slot1<void,int> );
    int TimeoutH(int number);

protected:

    //XSizeHints sizehints;
    //Gtk::Display *display;
    //int screen, rootwindow, defaultdepth, bwidth;
    char *geometry;
    char *appname;
    //GC NormalGC;
    GTKWindow window;
    Gdk_Pixmap shape;
    Gdk_Bitmap shapemask;
protected:
    int width, height;
    static int timeoutid;
    hash_map<int /*pid*/, ExecCallback> execs;

};

#endif
